/*
 * Copyright (C) 2009 HBM Netherlands B.V.
 * Schutweg 15a
 * 5145 NP Waalwijk
 * The Netherlands
 * http://www.hbm.com
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "GEN_DAQ_API.h"
#include "connection.h"
#include "jsonrpc.h"
#include "base64.h"
#include "fieldbus.h"

#define GHS_CLIENT_API_VERSION  4
#define NELEMENTS(array)        (sizeof (array) / sizeof ((array) [0]))
#define FIELDBUS_PORT 8007

#define MIN_BUFFER_SIZE 10

#define MIN(a,b) (((a)<(b))?(a):(b))
#define MAX(a,b) (((a)>(b))?(a):(b))

static void FreeJsonParams(JsonRpcNamedParameter *params, size_t numParams);
static char *stringDuplicate(const char *input);
static FieldBusHandle g_GenDaqFieldBusHandle = NULL;

GHSReturnValue GHSConnect(const char *ipAddress, unsigned short portNumber, GHSConnectionHandle *conHandle, uint32_t *serverAPIVersion)
{
    GHSReturnValue returnVal;
    JsonRpcNamedParameter *outParams = NULL;
    size_t nOutParams = 0;
    size_t i;

    if (!conHandle || !ipAddress || !serverAPIVersion)
        return GHSReturnValue_NullPtrArgument;

    /* Default value, in case the outputs are not found in our response */
    *conHandle = 0;
    *serverAPIVersion = 0;

    returnVal = ConnectionEstablish(conHandle, ipAddress, portNumber);
    if (returnVal == GHSReturnValue_OK)
    {
        JsonRpcNamedParameter inParams[1];
        const char *rpcRequest;
        uint16_t requestID = ConnectionNextRequestID(conHandle);

        inParams[0] = JsonRpcCreateIntNamedParameter("ClientAPIVersion", GHS_CLIENT_API_VERSION);

        rpcRequest = JsonRpcCreateRequest("Connect", requestID, inParams, NELEMENTS(inParams));
        returnVal = ConnectionSendRequestWaitResponse(conHandle, rpcRequest, requestID, &outParams, &nOutParams);

        /* ClientAPIVersion is sent even when connection fails. */
        for (i = 0; i < nOutParams; i++)
        {
            if (strcmp(outParams[i].key, "ServerAPIVersion") == 0)
                *serverAPIVersion = (uint32_t)outParams[i].value.dValue;
        }

        FreeJsonParams(outParams, nOutParams);

        if (returnVal != GHSReturnValue_OK)
            (void)ConnectionClose(conHandle);

        g_GenDaqFieldBusHandle = malloc(sizeof(FieldBus));
        g_GenDaqFieldBusHandle->dataHandle = NULL;
        g_GenDaqFieldBusHandle->conHandle = 0;
        g_GenDaqFieldBusHandle->LastPacket.fValues = NULL;

        g_GenDaqFieldBusHandle->ipAddress = (char *)malloc((strlen(ipAddress) + 1) * sizeof(char));
        strcpy(g_GenDaqFieldBusHandle->ipAddress, ipAddress);
    }

    return returnVal;
}

GHSReturnValue GHSDisconnect(GHSConnectionHandle conHandle)
{
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest = JsonRpcCreateRequest("Disconnect", requestID, NULL, 0);
    GHSReturnValue returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);

    (void)ConnectionClose(&conHandle);
    if(g_GenDaqFieldBusHandle && g_GenDaqFieldBusHandle->ipAddress && g_GenDaqFieldBusHandle)
    {
        free(g_GenDaqFieldBusHandle->ipAddress);
        free(g_GenDaqFieldBusHandle);
        g_GenDaqFieldBusHandle = NULL;
    }

    return returnVal;
}

uint32_t GHSGetClientAPIVersion(void)
{
    return (uint32_t)GHS_CLIENT_API_VERSION;
}

GHSReturnValue GHSGetCurrentAccess(GHSConnectionHandle conHandle, GHSAccess *access)
{
    uint16_t requestID;
    const char *rpcRequest;
    GHSReturnValue returnVal;
    JsonRpcNamedParameter *outParams = NULL;
    size_t nOutParams = 0;
    size_t i;

    if (!access)
        return GHSReturnValue_NullPtrArgument;

    requestID = ConnectionNextRequestID(&conHandle);
    rpcRequest = JsonRpcCreateRequest("GetCurrentAccess", requestID, NULL, 0);
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *access = GHSAccess_ReadOnly;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "Access") == 0)
            *access = (GHSAccess)outParams[i].value.dValue;
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSStartPreview(GHSConnectionHandle conHandle)
{
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest = JsonRpcCreateRequest("StartPreview", requestID, NULL, 0);
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSStopPreview(GHSConnectionHandle conHandle)
{
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest = JsonRpcCreateRequest("StopPreview", requestID, NULL, 0);
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSStartRecording(GHSConnectionHandle conHandle)
{
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest = JsonRpcCreateRequest("StartRecording", requestID, NULL, 0);
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSPauseRecording(GHSConnectionHandle conHandle)
{
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest = JsonRpcCreateRequest("PauseRecording", requestID, NULL, 0);
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSResumeRecording(GHSConnectionHandle conHandle)
{
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest = JsonRpcCreateRequest("ResumeRecording", requestID, NULL, 0);
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSStopRecording(GHSConnectionHandle conHandle)
{
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest = JsonRpcCreateRequest("StopRecording", requestID, NULL, 0);
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSTrigger(GHSConnectionHandle conHandle)
{
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest = JsonRpcCreateRequest("Trigger", requestID, NULL, 0);
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSGetAcquisitionState(GHSConnectionHandle conHandle, GHSAcquisitionState *state)
{
    uint16_t requestID;
    const char *rpcRequest;
    GHSReturnValue returnVal;
    JsonRpcNamedParameter *outParams = NULL;
    size_t nOutParams = 0;
    size_t i;

    if (!state)
        return GHSReturnValue_NullPtrArgument;

    requestID = ConnectionNextRequestID(&conHandle);
    rpcRequest = JsonRpcCreateRequest("GetAcquisitionState", requestID, NULL, 0);
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *state = GHSAcquisitionState_Reserved;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "GHSAcquisitionState") == 0)
            *state = (GHSAcquisitionState)outParams[i].value.dValue;
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSGetAcquisitionTime(GHSConnectionHandle conHandle, double *acquisitionTime)
{
    uint16_t requestID;
    const char *rpcRequest;
    GHSReturnValue returnVal;
    JsonRpcNamedParameter *outParams = NULL;
    size_t nOutParams = 0;
    size_t i;

    if (!acquisitionTime)
        return GHSReturnValue_NullPtrArgument;

    requestID = ConnectionNextRequestID(&conHandle);
    rpcRequest = JsonRpcCreateRequest("GetAcquisitionTime", requestID, NULL, 0);
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *acquisitionTime = -1;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "AcquisitionTime") == 0)
            *acquisitionTime = outParams[i].value.dValue;
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSGetAcquisitionStartTime(GHSConnectionHandle conHandle, int *year, int *day, double *seconds)
{
    uint16_t requestID;
    const char *rpcRequest;
    GHSReturnValue returnVal;
    JsonRpcNamedParameter *outParams = NULL;
    size_t nOutParams = 0;
    size_t i;

    if ((!year) || (!day) || (!seconds))
        return GHSReturnValue_NullPtrArgument;

    requestID = ConnectionNextRequestID(&conHandle);
    rpcRequest = JsonRpcCreateRequest("GetAcquisitionStartTime", requestID, NULL, 0);
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *year = -1;
    *day = -1;
    *seconds = -1;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "AbsoluteTimeYear") == 0)
            *year = (int)outParams[i].value.dValue;
        if (strcmp(outParams[i].key, "AbsoluteTimeDay") == 0)
            *day = (int)outParams[i].value.dValue;
        if (strcmp(outParams[i].key, "AbsoluteTimeSeconds") == 0)
            *seconds = outParams[i].value.dValue;
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSIdentify(GHSConnectionHandle conHandle, GHSEnableDisable enabled)
{
    JsonRpcNamedParameter inParams[1];
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest;

    inParams[0] = JsonRpcCreateIntNamedParameter("Identify", enabled);
    rpcRequest = JsonRpcCreateRequest("Identify", requestID, inParams, NELEMENTS(inParams));
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSGetDiskSpace(GHSConnectionHandle conHandle, double *total, double *available)
{
    GHSReturnValue returnVal;
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    JsonRpcNamedParameter *outParams = NULL;
    size_t nOutParams = 0;
    size_t i;
    const char *rpcRequest;

    if (!total || !available)
        return GHSReturnValue_NullPtrArgument;

    rpcRequest = JsonRpcCreateRequest("DiskSpace", requestID, NULL, 0);
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *total = 0.0;
    *available = 0.0;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "TotalSize") == 0)
            *total = outParams[i].value.dValue;
        if (strcmp(outParams[i].key, "AvailableSize") == 0)
            *available = outParams[i].value.dValue;
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSGetSyncStatus(GHSConnectionHandle conHandle, GHSSyncStatus *syncStatus)
{
    GHSReturnValue returnVal;
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    JsonRpcNamedParameter *outParams = NULL;
    size_t nOutParams = 0;
    size_t i;
    const char *rpcRequest;

    if (!syncStatus)
        return GHSReturnValue_NullPtrArgument;

    rpcRequest = JsonRpcCreateRequest("GetSyncStatus", requestID, NULL, 0);
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *syncStatus = GHSSyncStatus_Reserved;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "SyncStatus") == 0)
            *syncStatus = (GHSSyncStatus)outParams[i].value.dValue;
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSGetSlotCount(GHSConnectionHandle conHandle, int *slotCount)
{
    GHSReturnValue returnVal;
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    JsonRpcNamedParameter *outParams = NULL;
    size_t nOutParams = 0;
    size_t i;
    const char *rpcRequest;

    if (!slotCount)
        return GHSReturnValue_NullPtrArgument;

    rpcRequest = JsonRpcCreateRequest("GetSlotCount", requestID, NULL, 0);
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *slotCount = 0;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "SlotCount") == 0)
            *slotCount = (int)outParams[i].value.dValue;
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSGetUserMode(GHSConnectionHandle conHandle, GHSUserMode *userMode)
{
    GHSReturnValue returnVal;
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    JsonRpcNamedParameter *outParams = NULL;
    size_t nOutParams = 0;
    size_t i;
    const char *rpcRequest;

    if (!userMode)
        return GHSReturnValue_NullPtrArgument;

    rpcRequest = JsonRpcCreateRequest("GetUserMode", requestID, NULL, 0);
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *userMode = GHSUserMode_Reserved;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "UserMode") == 0)
            *userMode = (GHSUserMode)outParams[i].value.dValue;
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSSetUserMode(GHSConnectionHandle conHandle, GHSUserMode userMode)
{
    JsonRpcNamedParameter inParams[1];
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest;

    if (GHSUserMode_Sweeps != userMode && GHSUserMode_Continuous != userMode && GHSUserMode_Dual != userMode)
        return GHSReturnValue_InvalidUserMode;

    inParams[0] = JsonRpcCreateIntNamedParameter("UserMode", userMode);
    rpcRequest = JsonRpcCreateRequest("SetUserMode", requestID, inParams, NELEMENTS(inParams));
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSGetMainframeInformation(GHSConnectionHandle conHandle, char **mainframeType, char **mainframeName, char **serialNumber, char **firmwareVersion)
{
    uint16_t requestID;
    const char *rpcRequest;
    GHSReturnValue returnVal;
    JsonRpcNamedParameter *outParams = NULL;
    size_t nOutParams = 0;
    size_t i;

    if (!mainframeType || !mainframeName || !serialNumber || !firmwareVersion)
        return GHSReturnValue_NullPtrArgument;

    requestID = ConnectionNextRequestID(&conHandle);
    rpcRequest = JsonRpcCreateRequest("GetMainframeInformation", requestID, NULL, 0);
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *mainframeType = NULL;
    *mainframeName = NULL;
    *serialNumber = NULL;
    *firmwareVersion = NULL;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "MainframeType") == 0)
            *mainframeType = stringDuplicate(outParams[i].value.sValue);
        if (strcmp(outParams[i].key, "MainframeName") == 0)
            *mainframeName = stringDuplicate(outParams[i].value.sValue);
        if (strcmp(outParams[i].key, "SerialNumber") == 0)
            *serialNumber = stringDuplicate(outParams[i].value.sValue);
        if (strcmp(outParams[i].key, "FirmwareVersion") == 0)
            *firmwareVersion = stringDuplicate(outParams[i].value.sValue);
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSGetStorageLocation(GHSConnectionHandle conHandle, GHSStorageLocation *storageLocation)
{
    GHSReturnValue returnVal;

    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    JsonRpcNamedParameter *outParams = NULL;
    size_t nOutParams = 0;
    size_t i;
    const char *rpcRequest;

    if (!storageLocation)
        return GHSReturnValue_NullPtrArgument;

    rpcRequest = JsonRpcCreateRequest("GetStorageLocation", requestID, NULL, 0);
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *storageLocation = GHSStorageLocation_Reserved;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "StorageLocation") == 0)
            *storageLocation = (GHSStorageLocation)outParams[i].value.dValue;
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSSetStorageLocation(GHSConnectionHandle conHandle, GHSStorageLocation storageLocation)
{
    JsonRpcNamedParameter inParams[1];
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest;

    if (storageLocation == GHSStorageLocation_Reserved)
        return GHSReturnValue_IncompatibleStorageLocation;

    inParams[0] = JsonRpcCreateIntNamedParameter("StorageLocation", storageLocation);
    rpcRequest = JsonRpcCreateRequest("SetStorageLocation", requestID, inParams, NELEMENTS(inParams));
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSGetRecordingName(GHSConnectionHandle conHandle, const char **recordingBaseName, int *recordingIndex)
{
    uint16_t requestID;
    const char *rpcRequest;
    GHSReturnValue returnVal;
    JsonRpcNamedParameter *outParams = NULL;
    size_t nOutParams = 0;
    size_t i;

    if (!recordingBaseName || !recordingIndex)
        return GHSReturnValue_NullPtrArgument;

    requestID = ConnectionNextRequestID(&conHandle);
    rpcRequest = JsonRpcCreateRequest("GetRecordingName", requestID, NULL, 0);
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *recordingBaseName = NULL;
    *recordingIndex = 0;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "RecordingIndex") == 0)
            *recordingIndex = (int)outParams[i].value.dValue;

        if (strcmp(outParams[i].key, "RecordingName") == 0)
            *recordingBaseName = stringDuplicate(outParams[i].value.sValue);
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSSetRecordingName(GHSConnectionHandle conHandle, const char *recordingBaseName, int recordingIndex)
{
    const char *rpcRequest;
    JsonRpcNamedParameter params[2];
    uint16_t requestID = ConnectionNextRequestID(&conHandle);

    if (!recordingBaseName)
        return GHSReturnValue_NullPtrArgument;

    params[0] = JsonRpcCreateStringNamedParameter("RecordingBaseName", recordingBaseName);
    params[1] = JsonRpcCreateIntNamedParameter("RecordingIndex", recordingIndex);
    rpcRequest = JsonRpcCreateRequest("SetRecordingName", requestID, params, NELEMENTS(params));
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSSetRecordingBaseName(GHSConnectionHandle conHandle, const char *recordingBaseName)
{
    JsonRpcNamedParameter params[1];
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest;

    if (!recordingBaseName)
        return GHSReturnValue_NullPtrArgument;

    params[0] = JsonRpcCreateStringNamedParameter("RecordingBaseName", recordingBaseName);
    rpcRequest = JsonRpcCreateRequest("SetRecordingBaseName", requestID, params, NELEMENTS(params));
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSDeleteLastRecording(GHSConnectionHandle conHandle)
{
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest = JsonRpcCreateRequest("DeleteLastRecording", requestID, NULL, 0);
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSDeleteAllRecordings(GHSConnectionHandle conHandle)
{
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest = JsonRpcCreateRequest("DeleteAllRecordings", requestID, NULL, 0);
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSGetHighLowRateStorageEnabled(GHSConnectionHandle conHandle, GHSRecordingDataSource source, GHSSlotID slotID, GHSEnableDisable *highRateEnabled, GHSEnableDisable *lowRateEnabled)
{
    uint16_t requestID;
    const char *rpcRequest;
    GHSReturnValue returnVal;
    JsonRpcNamedParameter *outParams = NULL;
    char slotid[2];
    JsonRpcNamedParameter inParams[2];
    size_t nOutParams = 0;
    size_t i;

    if (!highRateEnabled || !lowRateEnabled)
        return GHSReturnValue_NullPtrArgument;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateIntNamedParameter("Source", source);
    inParams[1] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    requestID = ConnectionNextRequestID(&conHandle);
    rpcRequest = JsonRpcCreateRequest("GetHighLowRateStorageEnabled", requestID, inParams, NELEMENTS(inParams));
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *highRateEnabled = GHS_Disable;
    *lowRateEnabled = GHS_Disable;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "HighRateEnable") == 0)
            *highRateEnabled = (GHSEnableDisable)outParams[i].value.dValue;
        if (strcmp(outParams[i].key, "LowRateEnable") == 0)
            *lowRateEnabled = (GHSEnableDisable)outParams[i].value.dValue;
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSSetHighLowRateStorageEnabled(GHSConnectionHandle conHandle, GHSRecordingDataSource source, GHSSlotID slotID, GHSEnableDisable highRateEnabled, GHSEnableDisable lowRateEnabled)
{
    char slotid[2];
    JsonRpcNamedParameter inParams[4];
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateIntNamedParameter("Source", source);
    inParams[1] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    inParams[2] = JsonRpcCreateIntNamedParameter("HighRateEnable", highRateEnabled);
    inParams[3] = JsonRpcCreateIntNamedParameter("LowRateEnable", lowRateEnabled);
    rpcRequest = JsonRpcCreateRequest("SetHighLowRateStorageEnabled", requestID, inParams, NELEMENTS(inParams));
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSGetCurrentSettings(GHSConnectionHandle conHandle, GHSSettingsPtr *blob, size_t *blobSize)
{
    JsonRpcNamedParameter *outParams;
    size_t nOutParams;
    size_t i;
    uint16_t requestID;
    const char *rpcRequest;
    GHSReturnValue returnVal;

    if (!blob || !blobSize)
        return GHSReturnValue_NullPtrArgument;

    requestID = ConnectionNextRequestID(&conHandle);
    rpcRequest = JsonRpcCreateRequest("GetCurrentSettings", requestID, NULL, 0);
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *blob = NULL;
    *blobSize = 0;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "Size") == 0)
            *blobSize = (size_t)outParams[i].value.dValue;
        if (strcmp(outParams[i].key, "Blob") == 0)
        {
            const char *encoded = (outParams[i].value.sValue);
            size_t output_length = 0;

            *blob = Base64Decode(encoded, strlen(encoded), &output_length);
        }
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSSetCurrentSettings(GHSConnectionHandle conHandle, GHSSettingsPtr blob, size_t blobSize)
{
    const char *rpcRequest;
    JsonRpcNamedParameter inParams[2];
    size_t encodedBlobSize;
    char *pEncodedBlob;
    uint16_t requestID = ConnectionNextRequestID(&conHandle);

    if (!blob)
        return GHSReturnValue_NullPtrArgument;

    pEncodedBlob = Base64Encode(blob, blobSize, &encodedBlobSize);
    inParams[0] = JsonRpcCreateStringNamedParameter("Blob", pEncodedBlob);
    inParams[1] = JsonRpcCreateIntNamedParameter("Size", (int)blobSize);
    rpcRequest = JsonRpcCreateRequest("SetCurrentSettings", requestID, inParams, NELEMENTS(inParams));
    free(pEncodedBlob);

    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSFreeSettings(GHSSettingsPtr blob)
{
    if (!blob)
        return GHSReturnValue_NullPtrArgument;

    free(blob);
    return GHSReturnValue_OK;
}

GHSReturnValue GHSPersistCurrentSettings(GHSConnectionHandle conHandle)
{
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest = JsonRpcCreateRequest("PersistCurrentSettings", requestID, NULL, 0);
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSApplyPersistedSettings(GHSConnectionHandle conHandle)
{
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest = JsonRpcCreateRequest("ApplyPersistedSettings", requestID, NULL, 0);
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSGetRecorderInformation(GHSConnectionHandle conHandle, GHSSlotID id, char **recorderType, char **recorderName, char **serialNumber, char **firmwareVersion)
{
    char slotid[2];
    JsonRpcNamedParameter inParams[1];
    uint16_t requestID;
    const char *rpcRequest;
    GHSReturnValue returnVal;
    JsonRpcNamedParameter *outParams = NULL;
    size_t nOutParams = 0;
    size_t i;

    if (!recorderType || !recorderName || !serialNumber || !firmwareVersion)
        return GHSReturnValue_NullPtrArgument;

    slotid[0] = id;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    requestID = ConnectionNextRequestID(&conHandle);
    rpcRequest = JsonRpcCreateRequest("GetRecorderInformation", requestID, inParams, NELEMENTS(inParams));
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *recorderType = NULL;
    *recorderName = NULL;
    *serialNumber = NULL;
    *firmwareVersion = NULL;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "RecorderType") == 0)
            *recorderType = stringDuplicate(outParams[i].value.sValue);
        if (strcmp(outParams[i].key, "RecorderName") == 0)
            *recorderName = stringDuplicate(outParams[i].value.sValue);
        if (strcmp(outParams[i].key, "SerialNumber") == 0)
            *serialNumber = stringDuplicate(outParams[i].value.sValue);
        if (strcmp(outParams[i].key, "FirmwareVersion") == 0)
            *firmwareVersion = stringDuplicate(outParams[i].value.sValue);
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSGetRecorderEnabled(GHSConnectionHandle conHandle, GHSSlotID slotID, GHSEnableDisable *enabled)
{
    char slotid[2];
    JsonRpcNamedParameter inParams[1];
    uint16_t requestID;
    const char *rpcRequest;
    GHSReturnValue returnVal;
    JsonRpcNamedParameter *outParams = NULL;
    size_t nOutParams = 0;
    size_t i;

    if (!enabled)
        return GHSReturnValue_NullPtrArgument;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    requestID = ConnectionNextRequestID(&conHandle);
    rpcRequest = JsonRpcCreateRequest("GetRecorderEnabled", requestID, inParams, NELEMENTS(inParams));
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *enabled = GHS_Disable;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "IsRecorderEnabled") == 0)
            *enabled = (GHSEnableDisable)outParams[i].value.dValue;
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSSetRecorderEnabled(GHSConnectionHandle conHandle, GHSSlotID slotID, GHSEnableDisable enabled)
{
    char slotid[2];
    JsonRpcNamedParameter inParams[2];
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    inParams[1] = JsonRpcCreateIntNamedParameter("EnabledStatus", enabled);
    rpcRequest = JsonRpcCreateRequest("SetRecorderEnabled", requestID, inParams, NELEMENTS(inParams));
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSGetChannelCount(GHSConnectionHandle conHandle, GHSSlotID slotID, int *channelCount)
{
    char slotid[2];
    JsonRpcNamedParameter inParams[1];
    uint16_t requestID;
    const char *rpcRequest;
    GHSReturnValue returnVal;
    JsonRpcNamedParameter *outParams = NULL;
    size_t nOutParams = 0;
    size_t i;

    if (!channelCount)
        return GHSReturnValue_NullPtrArgument;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    requestID = ConnectionNextRequestID(&conHandle);
    rpcRequest = JsonRpcCreateRequest("GetChannelCount", requestID, inParams, NELEMENTS(inParams));
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *channelCount = 0;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "ChannelCount") == 0)
            *channelCount = (int)outParams[i].value.dValue;
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSGetSampleRate(GHSConnectionHandle conHandle, GHSSlotID id, double *sampleRate)
{
    char slotid[2];
    JsonRpcNamedParameter inParams[1];
    uint16_t requestID;
    const char *rpcRequest;
    GHSReturnValue returnVal;
    JsonRpcNamedParameter *outParams = NULL;
    size_t nOutParams = 0;
    size_t i;

    if (!sampleRate)
        return GHSReturnValue_NullPtrArgument;

    slotid[0] = id;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    requestID = ConnectionNextRequestID(&conHandle);
    rpcRequest = JsonRpcCreateRequest("GetSampleRate", requestID, inParams, NELEMENTS(inParams));
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *sampleRate = 0.0;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "SampleRate") == 0)
            *sampleRate = outParams[i].value.dValue;
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSSetSampleRate(GHSConnectionHandle conHandle, GHSSlotID id, double sampleRate)
{
    char slotid[2];
    JsonRpcNamedParameter inParams[2];
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest;

    slotid[0] = id;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    inParams[1] = JsonRpcCreateDoubleNamedParameter("SampleRate", sampleRate);
    rpcRequest = JsonRpcCreateRequest("SetSampleRate", requestID, inParams, NELEMENTS(inParams));
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSGetChannelType(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, GHSChannelType *channelType)
{
    char slotid[2];
    JsonRpcNamedParameter inParams[2];
    uint16_t requestID;
    const char *rpcRequest;
    GHSReturnValue returnVal;
    JsonRpcNamedParameter *outParams = NULL;
    size_t nOutParams = 0;
    size_t i;

    if (!channelType)
        return GHSReturnValue_NullPtrArgument;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    inParams[1] = JsonRpcCreateIntNamedParameter("ChannelIndex", channelIndex);
    requestID = ConnectionNextRequestID(&conHandle);
    rpcRequest = JsonRpcCreateRequest("GetChannelType", requestID, inParams, NELEMENTS(inParams));
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *channelType = GHSChannelType_Invalid;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "ChannelType") == 0)
            *channelType = (GHSChannelType)outParams[i].value.dValue;
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSGetChannelName(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, char **channelName)
{
    uint16_t requestID;
    const char *rpcRequest;
    GHSReturnValue returnVal;
    JsonRpcNamedParameter *outParams = NULL;
    char slotid[2];
    JsonRpcNamedParameter inParams[2];
    size_t nOutParams = 0;
    size_t i;

    if (!channelName)
        return GHSReturnValue_NullPtrArgument;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    inParams[1] = JsonRpcCreateIntNamedParameter("ChannelIndex", channelIndex);
    requestID = ConnectionNextRequestID(&conHandle);
    rpcRequest = JsonRpcCreateRequest("GetChannelName", requestID, inParams, NELEMENTS(inParams));
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *channelName = NULL;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "ChannelName") == 0)
            *channelName = stringDuplicate(outParams[i].value.sValue);
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSSetChannelName(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, const char *channelName)
{
    char slotid[2];
    JsonRpcNamedParameter inParams[3];
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    inParams[1] = JsonRpcCreateIntNamedParameter("ChannelIndex", channelIndex);
    inParams[2] = JsonRpcCreateStringNamedParameter("ChannelName", channelName);
    rpcRequest = JsonRpcCreateRequest("SetChannelName", requestID, inParams, NELEMENTS(inParams));
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSGetChannelStorageEnabled(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, GHSEnableDisable *enabled)
{
    char slotid[2];
    JsonRpcNamedParameter inParams[2];
    uint16_t requestID;
    const char *rpcRequest;
    GHSReturnValue returnVal;
    JsonRpcNamedParameter *outParams = NULL;
    size_t nOutParams = 0;
    size_t i;

    if (!enabled)
        return GHSReturnValue_NullPtrArgument;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    inParams[1] = JsonRpcCreateIntNamedParameter("ChannelIndex", channelIndex);
    requestID = ConnectionNextRequestID(&conHandle);
    rpcRequest = JsonRpcCreateRequest("GetChannelStorageEnabled", requestID, inParams, NELEMENTS(inParams));
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *enabled = GHS_Disable;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "Enabled") == 0)
            *enabled = (GHSEnableDisable)outParams[i].value.dValue;
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSSetChannelStorageEnabled(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, GHSEnableDisable enabled)
{
    char slotid[2];
    JsonRpcNamedParameter inParams[3];
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    inParams[1] = JsonRpcCreateIntNamedParameter("ChannelIndex", channelIndex);
    inParams[2] = JsonRpcCreateIntNamedParameter("ChannelStorageEnable", enabled);
    rpcRequest = JsonRpcCreateRequest("SetChannelStorageEnabled", requestID, inParams, NELEMENTS(inParams));
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSGetTriggerSettings(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, GHSTriggerMode *triggerMode, double *primaryLevel, double *secondaryLevel, double *hysteresis, GHSDirection *direction)
{
    uint16_t requestID;
    const char *rpcRequest;
    GHSReturnValue returnVal;
    JsonRpcNamedParameter *outParams = NULL;
    char slotid[2];
    JsonRpcNamedParameter inParams[2];
    size_t nOutParams = 0;
    size_t i;

    if (!triggerMode || !primaryLevel ||!secondaryLevel || !hysteresis ||!direction)
        return GHSReturnValue_NullPtrArgument;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    inParams[1] = JsonRpcCreateIntNamedParameter("ChannelIndex", channelIndex);
    requestID = ConnectionNextRequestID(&conHandle);
    rpcRequest = JsonRpcCreateRequest("GetTriggerSettings", requestID, inParams, NELEMENTS(inParams));
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *triggerMode = GHSTriggerMode_Off;
    *primaryLevel = 0.0;
    *secondaryLevel = 0.0;
    *hysteresis = 0.0;
    *direction = GHSDirection_RisingEdge;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "TriggerMode") == 0)
            *triggerMode = (GHSTriggerMode)outParams[i].value.dValue;
        if (strcmp(outParams[i].key, "PrimaryLevel") == 0)
            *primaryLevel = outParams[i].value.dValue;
        if (strcmp(outParams[i].key, "SecondaryLevel") == 0)
            *secondaryLevel = outParams[i].value.dValue;
        if (strcmp(outParams[i].key, "Hysteresis") == 0)
            *hysteresis = outParams[i].value.dValue;
        if (strcmp(outParams[i].key, "Direction") == 0)
            *direction = (GHSDirection)outParams[i].value.dValue;
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSSetTriggerSettings(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, GHSTriggerMode triggerMode, double primaryLevel, double secondaryLevel, double hysteresis, GHSDirection direction)
{
    char slotid[2];
    JsonRpcNamedParameter inParams[7];
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    inParams[1] = JsonRpcCreateIntNamedParameter("ChannelIndex", channelIndex);
    inParams[2] = JsonRpcCreateIntNamedParameter("TriggerMode", triggerMode);
    inParams[3] = JsonRpcCreateDoubleNamedParameter("PrimaryLevel", primaryLevel);
    inParams[4] = JsonRpcCreateDoubleNamedParameter("SecondaryLevel", secondaryLevel);
    inParams[5] = JsonRpcCreateDoubleNamedParameter("Hysteresis", hysteresis);
    inParams[6] = JsonRpcCreateIntNamedParameter("Direction", direction);
    rpcRequest = JsonRpcCreateRequest("SetTriggerSettings", requestID, inParams, NELEMENTS(inParams));
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSGetSignalCoupling(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, GHSSignalCoupling *signalCoupling)
{
    uint16_t requestID;
    const char *rpcRequest;
    GHSReturnValue returnVal;
    JsonRpcNamedParameter *outParams = NULL;
    char slotid[2];
    JsonRpcNamedParameter inParams[2];
    size_t nOutParams = 0;
    size_t i;

    if (!signalCoupling)
        return GHSReturnValue_NullPtrArgument;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    inParams[1] = JsonRpcCreateIntNamedParameter("ChannelIndex", channelIndex);
    requestID = ConnectionNextRequestID(&conHandle);
    rpcRequest = JsonRpcCreateRequest("GetSignalCoupling", requestID, inParams, NELEMENTS(inParams));
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *signalCoupling = GHSSigCpl_GND;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "SignalCoupling") == 0)
            *signalCoupling = (GHSSignalCoupling)outParams[i].value.dValue;
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSSetSignalCoupling(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, GHSSignalCoupling signalCoupling)
{
    char slotid[2];
    JsonRpcNamedParameter inParams[3];
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    inParams[1] = JsonRpcCreateIntNamedParameter("ChannelIndex", channelIndex);
    inParams[2] = JsonRpcCreateIntNamedParameter("SignalCoupling", signalCoupling);
    rpcRequest = JsonRpcCreateRequest("SetSignalCoupling", requestID, inParams, NELEMENTS(inParams));
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSGetInputCoupling(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, GHSInputCoupling *inputCoupling)
{
    uint16_t requestID;
    const char *rpcRequest;
    GHSReturnValue returnVal;
    JsonRpcNamedParameter *outParams = NULL;
    char slotid[2];
    JsonRpcNamedParameter inParams[2];
    size_t nOutParams = 0;
    size_t i;

    if (!inputCoupling)
        return GHSReturnValue_NullPtrArgument;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    inParams[1] = JsonRpcCreateIntNamedParameter("ChannelIndex", channelIndex);
    requestID = ConnectionNextRequestID(&conHandle);
    rpcRequest = JsonRpcCreateRequest("GetInputCoupling", requestID, inParams, NELEMENTS(inParams));
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *inputCoupling = GHSInCpl_SingleEndedPositive;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "InputCoupling") == 0)
            *inputCoupling = (GHSInputCoupling)outParams[i].value.dValue;
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSSetInputCoupling(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, GHSInputCoupling inputCoupling)
{
    char slotid[2];
    JsonRpcNamedParameter inParams[3];
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    inParams[1] = JsonRpcCreateIntNamedParameter("ChannelIndex", channelIndex);
    inParams[2] = JsonRpcCreateIntNamedParameter("InputCoupling", inputCoupling);
    rpcRequest = JsonRpcCreateRequest("SetInputCoupling", requestID, inParams, NELEMENTS(inParams));
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSGetSpanAndOffset(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, double *span, double *offset)
{
    uint16_t requestID;
    const char *rpcRequest;
    GHSReturnValue returnVal;
    JsonRpcNamedParameter *outParams = NULL;
    char slotid[2];
    JsonRpcNamedParameter inParams[2];
    size_t nOutParams = 0;
    size_t i;

    if (!span || !offset)
        return GHSReturnValue_NullPtrArgument;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    inParams[1] = JsonRpcCreateIntNamedParameter("ChannelIndex", channelIndex);
    requestID = ConnectionNextRequestID(&conHandle);
    rpcRequest = JsonRpcCreateRequest("GetSpanAndOffset", requestID, inParams, NELEMENTS(inParams));
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *span = 0.0;
    *offset = 0.0;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "Span") == 0)
            *span = outParams[i].value.dValue;
        if (strcmp(outParams[i].key, "Offset") == 0)
            *offset = outParams[i].value.dValue;
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSSetSpanAndOffset(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, double span, double offset)
{
    char slotid[2];
    JsonRpcNamedParameter inParams[4];
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    inParams[1] = JsonRpcCreateIntNamedParameter("ChannelIndex", channelIndex);
    inParams[2] = JsonRpcCreateDoubleNamedParameter("Span", span);
    inParams[3] = JsonRpcCreateDoubleNamedParameter("Offset", offset);
    rpcRequest = JsonRpcCreateRequest("SetSpanAndOffset", requestID, inParams, NELEMENTS(inParams));
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSGetFilterTypeAndFrequency(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, GHSFilterType *filterType, double *frequency)
{
    uint16_t requestID;
    const char *rpcRequest;
    GHSReturnValue returnVal;
    JsonRpcNamedParameter *outParams = NULL;
    char slotid[2];
    JsonRpcNamedParameter inParams[2];
    size_t nOutParams = 0;
    size_t i;

    if (!filterType || !frequency)
        return GHSReturnValue_NullPtrArgument;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    inParams[1] = JsonRpcCreateIntNamedParameter("ChannelIndex", channelIndex);
    requestID = ConnectionNextRequestID(&conHandle);
    rpcRequest = JsonRpcCreateRequest("GetFilterTypeAndFrequency", requestID, inParams, NELEMENTS(inParams));
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *filterType = GHSFilterType_Bessel;
    *frequency = 0.0;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "FilterType") == 0)
            *filterType = (GHSFilterType)outParams[i].value.dValue;
        if (strcmp(outParams[i].key, "Frequency") == 0)
            *frequency = outParams[i].value.dValue;
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSSetFilterTypeAndFrequency(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, GHSFilterType filterType, double frequency)
{
    char slotid[2];
    JsonRpcNamedParameter inParams[4];
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    inParams[1] = JsonRpcCreateIntNamedParameter("ChannelIndex", channelIndex);
    inParams[2] = JsonRpcCreateIntNamedParameter("FilterType", filterType);
    inParams[3] = JsonRpcCreateDoubleNamedParameter("Frequency", frequency);
    rpcRequest = JsonRpcCreateRequest("SetFilterTypeAndFrequency", requestID, inParams, NELEMENTS(inParams));
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSGetExcitation(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, GHSExcitationType *excitationType, double *excitationValue)
{
    uint16_t requestID;
    const char *rpcRequest;
    GHSReturnValue returnVal;
    JsonRpcNamedParameter *outParams = NULL;
    char slotid[2];
    JsonRpcNamedParameter inParams[2];
    size_t nOutParams = 0;
    size_t i;

    if (!excitationType || !excitationValue)
        return GHSReturnValue_NullPtrArgument;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    inParams[1] = JsonRpcCreateIntNamedParameter("ChannelIndex", channelIndex);
    requestID = ConnectionNextRequestID(&conHandle);
    rpcRequest = JsonRpcCreateRequest("GetExcitation", requestID, inParams, NELEMENTS(inParams));
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *excitationType = GHSExcitationType_Voltage;
    *excitationValue = 0.0;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "ExcitationType") == 0)
            *excitationType = (GHSExcitationType)outParams[i].value.dValue;
        if (strcmp(outParams[i].key, "ExcitationValue") == 0)
            *excitationValue = outParams[i].value.dValue;
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSSetExcitation(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, GHSExcitationType excitationType, double excitationValue)
{
    char slotid[2];
    JsonRpcNamedParameter inParams[4];
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    inParams[1] = JsonRpcCreateIntNamedParameter("ChannelIndex", channelIndex);
    inParams[2] = JsonRpcCreateIntNamedParameter("ExcitationType", excitationType);
    inParams[3] = JsonRpcCreateDoubleNamedParameter("ExcitationValue", excitationValue);
    rpcRequest = JsonRpcCreateRequest("SetExcitation", requestID, inParams, NELEMENTS(inParams));
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSGetAmplifierMode(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, GHSAmplifierMode *amplifierMode)
{
    uint16_t requestID;
    const char *rpcRequest;
    GHSReturnValue returnVal;
    JsonRpcNamedParameter *outParams = NULL;
    char slotid[2];
    JsonRpcNamedParameter inParams[2];
    size_t nOutParams = 0;
    size_t i;

    if (!amplifierMode)
        return GHSReturnValue_NullPtrArgument;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    inParams[1] = JsonRpcCreateIntNamedParameter("ChannelIndex", channelIndex);
    requestID = ConnectionNextRequestID(&conHandle);
    rpcRequest = JsonRpcCreateRequest("GetAmplifierMode", requestID, inParams, NELEMENTS(inParams));
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *amplifierMode = GHSAmplifierMode_Basic;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "AmplifierMode") == 0)
            *amplifierMode = (GHSAmplifierMode)outParams[i].value.dValue;
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSSetAmplifierMode(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, GHSAmplifierMode amplifierMode)
{
    char slotid[2];
    JsonRpcNamedParameter inParams[3];
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    inParams[1] = JsonRpcCreateIntNamedParameter("ChannelIndex", channelIndex);
    inParams[2] = JsonRpcCreateIntNamedParameter("AmplifierMode", amplifierMode);
    rpcRequest = JsonRpcCreateRequest("SetAmplifierMode", requestID, inParams, NELEMENTS(inParams));
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSGetTechnicalUnits(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, char **units, double *multiplier, double *offset)
{
    uint16_t requestID;
    const char *rpcRequest;
    GHSReturnValue returnVal;
    JsonRpcNamedParameter *outParams = NULL;
    char slotid[2];
    JsonRpcNamedParameter inParams[2];
    size_t nOutParams = 0;
    size_t i;

    if (!units || !multiplier || !offset)
        return GHSReturnValue_NullPtrArgument;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    inParams[1] = JsonRpcCreateIntNamedParameter("ChannelIndex", channelIndex);
    requestID = ConnectionNextRequestID(&conHandle);
    rpcRequest = JsonRpcCreateRequest("GetTechnicalUnits", requestID, inParams, NELEMENTS(inParams));
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *multiplier = 0.0;
    *offset = 0.0;
    *units = NULL;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "Multiplier") == 0)
            *multiplier = outParams[i].value.dValue;
        if (strcmp(outParams[i].key, "Offset") == 0)
            *offset = outParams[i].value.dValue;
        if (strcmp(outParams[i].key, "UnitType") == 0)
            *units = stringDuplicate(outParams[i].value.sValue);
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSSetTechnicalUnits(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, const char *units, double multiplier, double offset)
{
    char slotid[2];
    JsonRpcNamedParameter inParams[5];
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    inParams[1] = JsonRpcCreateIntNamedParameter("ChannelIndex", channelIndex);
    inParams[2] = JsonRpcCreateStringNamedParameter("UnitType", units);
    inParams[3] = JsonRpcCreateDoubleNamedParameter("Multiplier", multiplier);
    inParams[4] = JsonRpcCreateDoubleNamedParameter("Offset", offset);
    rpcRequest = JsonRpcCreateRequest("SetTechnicalUnits", requestID, inParams, NELEMENTS(inParams));
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSGetAutoRange(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, GHSEnableDisable *autoRangeEnabled, double *autoRangeTime)
{
    uint16_t requestID;
    const char *rpcRequest;
    GHSReturnValue returnVal;
    JsonRpcNamedParameter *outParams = NULL;
    char slotid[2];
    JsonRpcNamedParameter inParams[2];
    size_t nOutParams = 0;
    size_t i;

    if (!autoRangeEnabled || !autoRangeTime)
        return GHSReturnValue_NullPtrArgument;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    inParams[1] = JsonRpcCreateIntNamedParameter("ChannelIndex", channelIndex);
    requestID = ConnectionNextRequestID(&conHandle);
    rpcRequest = JsonRpcCreateRequest("GetAutoRange", requestID, inParams, NELEMENTS(inParams));
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *autoRangeEnabled = 0;
    *autoRangeTime = 0;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "AutoRangeEnabled") == 0)
            *autoRangeEnabled = (GHSEnableDisable)outParams[i].value.dValue;
        if (strcmp(outParams[i].key, "AutoRangeTime") == 0)
            *autoRangeTime = outParams[i].value.dValue;
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSSetAutoRange(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, GHSEnableDisable AutoRangeEnabled, double AutoRangeTime)
{
    char slotid[2];
    JsonRpcNamedParameter inParams[4];
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    inParams[1] = JsonRpcCreateIntNamedParameter("ChannelIndex", channelIndex);
    inParams[2] = JsonRpcCreateIntNamedParameter("AutoRangeEnabled", AutoRangeEnabled);
    inParams[3] = JsonRpcCreateDoubleNamedParameter("AutoRangeTime", AutoRangeTime);
    rpcRequest = JsonRpcCreateRequest("SetAutoRange", requestID, inParams, NELEMENTS(inParams));
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSCmdAutoRangeNow(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, double AutoRangeTime)
{
    char slotid[2];
    JsonRpcNamedParameter inParams[3];
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    inParams[1] = JsonRpcCreateIntNamedParameter("ChannelIndex", channelIndex);
    inParams[2] = JsonRpcCreateDoubleNamedParameter("AutoRangeTime", AutoRangeTime);
    rpcRequest = JsonRpcCreateRequest("AutoRangeNow", requestID, inParams, NELEMENTS(inParams));
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSGetChannelCalibrationInformation(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, char** calibrationDateTime, char** verificationDateTime, char** powerVerificationDateTime, char** calibrationLab, char** verificationLab, char** powerVerificationLab)
{
    char slotid[2];
    JsonRpcNamedParameter inParams[2];
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char* rpcRequest;
    GHSReturnValue returnVal;
    JsonRpcNamedParameter* outParams = NULL;
    size_t nOutParams = 0;
    size_t i;

    if (!calibrationDateTime || !verificationDateTime || !powerVerificationDateTime || !calibrationLab || !verificationLab || !powerVerificationLab)
        return GHSReturnValue_NullPtrArgument;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    inParams[1] = JsonRpcCreateIntNamedParameter("ChannelIndex", channelIndex);
    rpcRequest = JsonRpcCreateRequest("GetChannelCalibrationInformation", requestID, inParams, NELEMENTS(inParams));
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *calibrationDateTime = NULL;
    *verificationDateTime = NULL;
    *powerVerificationDateTime = NULL;
    *calibrationLab = NULL;
    *verificationLab = NULL;
    *powerVerificationLab = NULL;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "CalibrationDateTime") == 0)
            *calibrationDateTime = stringDuplicate(outParams[i].value.sValue);
        if (strcmp(outParams[i].key, "VerificationDateTime") == 0)
            *verificationDateTime = stringDuplicate(outParams[i].value.sValue);
        if (strcmp(outParams[i].key, "PowerVerificationDateTime") == 0)
            *powerVerificationDateTime = stringDuplicate(outParams[i].value.sValue);
        if (strcmp(outParams[i].key, "CalibrationLab") == 0)
            *calibrationLab = stringDuplicate(outParams[i].value.sValue);
        if (strcmp(outParams[i].key, "VerificationLab") == 0)
            *verificationLab = stringDuplicate(outParams[i].value.sValue);
        if (strcmp(outParams[i].key, "PowerVerificationLab") == 0)
            *powerVerificationLab = stringDuplicate(outParams[i].value.sValue);
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSGetTimerCounterMode(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, GHSTimerCounterMode *mode)
{
    uint16_t requestID;
    const char *rpcRequest;
    GHSReturnValue returnVal;
    JsonRpcNamedParameter *outParams = NULL;
    char slotid[2];
    JsonRpcNamedParameter inParams[2];
    size_t nOutParams = 0;
    size_t i;

    if (!mode)
        return GHSReturnValue_NullPtrArgument;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    inParams[1] = JsonRpcCreateIntNamedParameter("ChannelIndex", channelIndex);
    requestID = ConnectionNextRequestID(&conHandle);
    rpcRequest = JsonRpcCreateRequest("GetTimerCounterMode", requestID, inParams, NELEMENTS(inParams));
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *mode = GHSTimerCounterMode_RPMUniDirectional;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "TimerCounterMode") == 0)
            *mode = (GHSTimerCounterMode)outParams[i].value.dValue;
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSSetTimerCounterMode(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, GHSTimerCounterMode mode)
{
    char slotid[2];
    JsonRpcNamedParameter inParams[3];
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    inParams[1] = JsonRpcCreateIntNamedParameter("ChannelIndex", channelIndex);
    inParams[2] = JsonRpcCreateIntNamedParameter("TimerCounterMode", mode);
    rpcRequest = JsonRpcCreateRequest("SetTimerCounterMode", requestID, inParams, NELEMENTS(inParams));
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSGetTimerCounterGateTime(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, double *gateTime)
{
    uint16_t requestID;
    const char *rpcRequest;
    GHSReturnValue returnVal;
    JsonRpcNamedParameter *outParams = NULL;
    char slotid[2];
    JsonRpcNamedParameter inParams[2];
    size_t nOutParams = 0;
    size_t i;

    if (!gateTime)
        return GHSReturnValue_NullPtrArgument;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    inParams[1] = JsonRpcCreateIntNamedParameter("ChannelIndex", channelIndex);
    requestID = ConnectionNextRequestID(&conHandle);
    rpcRequest = JsonRpcCreateRequest("GetTimerCounterGateTime", requestID, inParams, NELEMENTS(inParams));
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *gateTime = 0;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "GateTime") == 0)
            *gateTime = outParams[i].value.dValue;
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSSetTimerCounterGateTime(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, double gateTime)
{
    char slotid[2];
    JsonRpcNamedParameter inParams[3];
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    inParams[1] = JsonRpcCreateIntNamedParameter("ChannelIndex", channelIndex);
    inParams[2] = JsonRpcCreateDoubleNamedParameter("GateTime", gateTime);
    rpcRequest = JsonRpcCreateRequest("SetTimerCounterGateTime", requestID, inParams, NELEMENTS(inParams));
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSGetTimerCounterRange(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, double *lowerValue, double *upperValue)
{
    uint16_t requestID;
    const char *rpcRequest;
    GHSReturnValue returnVal;
    JsonRpcNamedParameter *outParams = NULL;
    char slotid[2];
    JsonRpcNamedParameter inParams[2];
    size_t nOutParams = 0;
    size_t i;

    if (!lowerValue || !upperValue)
        return GHSReturnValue_NullPtrArgument;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    inParams[1] = JsonRpcCreateIntNamedParameter("ChannelIndex", channelIndex);
    requestID = ConnectionNextRequestID(&conHandle);
    rpcRequest = JsonRpcCreateRequest("GetTimerCounterRange", requestID, inParams, NELEMENTS(inParams));
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *lowerValue = 0.0;
    *upperValue = 0.0;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "UpperValue") == 0)
            *upperValue = outParams[i].value.dValue;
        if (strcmp(outParams[i].key, "LowerValue") == 0)
            *lowerValue = outParams[i].value.dValue;
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSSetTimerCounterRange(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, double lowerValue, double upperValue)
{
    char slotid[2];
    JsonRpcNamedParameter inParams[4];
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    inParams[1] = JsonRpcCreateIntNamedParameter("ChannelIndex", channelIndex);
    inParams[2] = JsonRpcCreateDoubleNamedParameter("LowerValue", lowerValue);
    inParams[3] = JsonRpcCreateDoubleNamedParameter("UpperValue", upperValue);
    rpcRequest = JsonRpcCreateRequest("SetTimerCounterRange", requestID, inParams, NELEMENTS(inParams));
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSGetSweepRecordingMode(GHSConnectionHandle conHandle, GHSSlotID slotID, GHSSweepRecordingMode *sweepRecordingMode)
{
    char slotid[2];
    JsonRpcNamedParameter inParams[1];
    uint16_t requestID;
    const char *rpcRequest;
    GHSReturnValue returnVal;
    JsonRpcNamedParameter *outParams = NULL;
    size_t nOutParams = 0;
    size_t i;

    if (!sweepRecordingMode)
        return GHSReturnValue_NullPtrArgument;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    requestID = ConnectionNextRequestID(&conHandle);
    rpcRequest = JsonRpcCreateRequest("GetSweepMode", requestID, inParams, NELEMENTS(inParams));
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *sweepRecordingMode = GHSSweepMode_Reserved;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "SweepMode") == 0)
            *sweepRecordingMode = (GHSSweepRecordingMode)outParams[i].value.dValue;
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSSetSweepRecordingMode(GHSConnectionHandle conHandle, GHSSlotID slotID, GHSSweepRecordingMode sweepRecordingMode)
{
    char slotid[2];
    JsonRpcNamedParameter inParams[2];
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest;

    if (GHSSweepMode_Normal != sweepRecordingMode && GHSSweepMode_PreTrigger != sweepRecordingMode)
        return GHSReturnValue_InvalidSweepMode;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    inParams[1] = JsonRpcCreateDoubleNamedParameter("SweepMode", sweepRecordingMode);
    rpcRequest = JsonRpcCreateRequest("SetSweepMode", requestID, inParams, NELEMENTS(inParams));
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSGetSweepLength(GHSConnectionHandle conHandle, GHSSlotID slotID, int64_t *numberOfSamples)
{
    char slotid[2];
    JsonRpcNamedParameter inParams[1];
    uint16_t requestID;
    const char *rpcRequest;
    GHSReturnValue returnVal;
    JsonRpcNamedParameter *outParams = NULL;
    size_t nOutParams = 0;
    size_t i;

    if (!numberOfSamples)
        return GHSReturnValue_NullPtrArgument;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    requestID = ConnectionNextRequestID(&conHandle);
    rpcRequest = JsonRpcCreateRequest("GetSweepLength", requestID, inParams, NELEMENTS(inParams));
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *numberOfSamples = 0;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "SweepLength") == 0)
            *numberOfSamples =(int64_t) outParams[i].value.dValue;
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSSetSweepLength(GHSConnectionHandle conHandle, GHSSlotID slotID, int64_t numberOfSamples)
{
    char slotid[2];
    JsonRpcNamedParameter inParams[2];
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    inParams[1] = JsonRpcCreateDoubleNamedParameter("SweepLength", (double)numberOfSamples);
    rpcRequest = JsonRpcCreateRequest("SetSweepLength", requestID, inParams, NELEMENTS(inParams));
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSGetNumberOfSweeps(GHSConnectionHandle conHandle, GHSSlotID slotID, int *numberOfSweeps)
{
    char slotid[2];
    JsonRpcNamedParameter inParams[1];
    uint16_t requestID;
    const char *rpcRequest;
    GHSReturnValue returnVal;
    JsonRpcNamedParameter *outParams = NULL;
    size_t nOutParams = 0;
    size_t i;

    if (!numberOfSweeps)
        return GHSReturnValue_NullPtrArgument;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    requestID = ConnectionNextRequestID(&conHandle);
    rpcRequest = JsonRpcCreateRequest("GetNumberOfSweeps", requestID, inParams, NELEMENTS(inParams));
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *numberOfSweeps = 0;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "NumberSweeps") == 0)
            *numberOfSweeps = (int)outParams[i].value.dValue;
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSSetNumberOfSweeps(GHSConnectionHandle conHandle, GHSSlotID slotID, int numberOfSweeps)
{
    char slotid[2];
    JsonRpcNamedParameter inParams[2];
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    inParams[1] = JsonRpcCreateDoubleNamedParameter("NumberSweeps", numberOfSweeps);
    rpcRequest = JsonRpcCreateRequest("SetNumberOfSweeps", requestID, inParams, NELEMENTS(inParams));
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSGetTriggerPosition(GHSConnectionHandle conHandle, GHSSlotID slotID, double *triggerPosition)
{
    char slotid[2];
    JsonRpcNamedParameter inParams[1];
    uint16_t requestID;
    const char *rpcRequest;
    GHSReturnValue returnVal;
    JsonRpcNamedParameter *outParams = NULL;
    size_t nOutParams = 0;
    size_t i;

    if (!triggerPosition)
        return GHSReturnValue_NullPtrArgument;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    requestID = ConnectionNextRequestID(&conHandle);
    rpcRequest = JsonRpcCreateRequest("GetTriggerPosition", requestID, inParams, NELEMENTS(inParams));
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *triggerPosition = 0.0;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "TriggerPosition") == 0)
            *triggerPosition = outParams[i].value.dValue;
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSSetTriggerPosition(GHSConnectionHandle conHandle, GHSSlotID slotID, double triggerPosition)
{
    char slotid[2];
    JsonRpcNamedParameter inParams[2];
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest;

    if (triggerPosition < 0 || triggerPosition > 100)
        return GHSReturnValue_InvalidTriggerPosition;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    inParams[1] = JsonRpcCreateDoubleNamedParameter("TriggerPosition", triggerPosition);
    rpcRequest = JsonRpcCreateRequest("SetTriggerPosition", requestID, inParams, NELEMENTS(inParams));
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSGetSweepStretchEnabled(GHSConnectionHandle conHandle, GHSSlotID slotID, GHSEnableDisable *enabled)
{
    char slotid[2];
    JsonRpcNamedParameter inParams[1];
    uint16_t requestID;
    const char *rpcRequest;
    GHSReturnValue returnVal;
    JsonRpcNamedParameter *outParams = NULL;
    size_t nOutParams = 0;
    size_t i;

    if (!enabled)
        return GHSReturnValue_NullPtrArgument;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    requestID = ConnectionNextRequestID(&conHandle);
    rpcRequest = JsonRpcCreateRequest("GetSweepStretchEnabled", requestID, inParams, NELEMENTS(inParams));
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *enabled = GHS_Disable;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "SweepStretchStatus") == 0)
            *enabled = (GHSEnableDisable)outParams[i].value.dValue;
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSSetSweepStretchEnabled(GHSConnectionHandle conHandle, GHSSlotID slotID, GHSEnableDisable enabled)
{
    char slotid[2];
    JsonRpcNamedParameter inParams[2];
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    inParams[1] = JsonRpcCreateDoubleNamedParameter("SweepStretchStatus", enabled);
    rpcRequest = JsonRpcCreateRequest("SetSweepStretchEnabled", requestID, inParams, NELEMENTS(inParams));
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSGetTriggerArmEnabled(GHSConnectionHandle conHandle, GHSEnableDisable *enabled)
{
    uint16_t requestID;
    const char *rpcRequest;
    GHSReturnValue returnVal;
    JsonRpcNamedParameter *outParams = NULL;
    size_t nOutParams = 0;
    size_t i;

    if (!enabled)
        return GHSReturnValue_NullPtrArgument;

    requestID = ConnectionNextRequestID(&conHandle);
    rpcRequest = JsonRpcCreateRequest("GetTriggerArmEnabled", requestID, NULL, 0);
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *enabled = GHS_Disable;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "Enable") == 0)
            *enabled = (GHSEnableDisable)outParams[i].value.dValue;
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSSetTriggerArmEnabled(GHSConnectionHandle conHandle, GHSEnableDisable enabled)
{
    JsonRpcNamedParameter inParams[1];
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest;

    inParams[0] = JsonRpcCreateIntNamedParameter("Enable", enabled);
    rpcRequest = JsonRpcCreateRequest("SetTriggerArmEnabled", requestID, inParams, NELEMENTS(inParams));
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSCmdTriggerArm(GHSConnectionHandle conHandle)
{
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest = JsonRpcCreateRequest("TriggerArm", requestID, NULL, 0);
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSGetTriggerArmState(GHSConnectionHandle conHandle, GHSTriggerArmState* state)
{
    uint16_t requestID;
    const char* rpcRequest;
    GHSReturnValue returnVal;
    JsonRpcNamedParameter* outParams = NULL;
    size_t nOutParams = 0;
    size_t i;

    if (!state)
        return GHSReturnValue_NullPtrArgument;

    requestID = ConnectionNextRequestID(&conHandle);
    rpcRequest = JsonRpcCreateRequest("GetTriggerArmState", requestID, NULL, 0);
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *state = GHSTriggerArmState_Armed;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "TriggerArmState") == 0)
            *state = (GHSTriggerArmState)outParams[i].value.dValue;
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSGetSweepTriggerMode(GHSConnectionHandle conHandle, GHSSlotID slotID, GHSSweepTriggerMode *mode)
{
    char slotid[2];
    JsonRpcNamedParameter inParams[1];
    uint16_t requestID;
    const char *rpcRequest;
    GHSReturnValue returnVal;
    JsonRpcNamedParameter *outParams = NULL;
    size_t nOutParams = 0;
    size_t i;

    if (!mode)
        return GHSReturnValue_NullPtrArgument;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    requestID = ConnectionNextRequestID(&conHandle);
    rpcRequest = JsonRpcCreateRequest("GetSweepTriggerMode", requestID, inParams, NELEMENTS(inParams));
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *mode = GHSSweepTriggerMode_FirstTrigger;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "SweepTriggerMode") == 0)
            *mode = (GHSSweepTriggerMode)outParams[i].value.dValue;
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSSetSweepTriggerMode(GHSConnectionHandle conHandle, GHSSlotID slotID, GHSSweepTriggerMode mode)
{
    char slotid[2];
    JsonRpcNamedParameter inParams[2];
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    inParams[1] = JsonRpcCreateIntNamedParameter("SweepTriggerMode", mode);
    rpcRequest = JsonRpcCreateRequest("SetSweepTriggerMode", requestID, inParams, NELEMENTS(inParams));
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSGetTimeoutTriggerEnabled(GHSConnectionHandle conHandle, GHSEnableDisable *enabled)
{
    uint16_t requestID;
    const char *rpcRequest;
    GHSReturnValue returnVal;
    JsonRpcNamedParameter *outParams = NULL;
    size_t nOutParams = 0;
    size_t i;

    if (!enabled)
        return GHSReturnValue_NullPtrArgument;

    requestID = ConnectionNextRequestID(&conHandle);
    rpcRequest = JsonRpcCreateRequest("GetTimeoutTriggerEnabled", requestID, NULL, 0);
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *enabled = GHS_Disable;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "Enable") == 0)
            *enabled = (GHSEnableDisable)outParams[i].value.dValue;
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSSetTimeoutTriggerEnabled(GHSConnectionHandle conHandle, GHSEnableDisable enabled)
{
    JsonRpcNamedParameter inParams[1];
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest;

    inParams[0] = JsonRpcCreateIntNamedParameter("Enable", enabled);
    rpcRequest = JsonRpcCreateRequest("SetTimeoutTriggerEnabled", requestID, inParams, NELEMENTS(inParams));
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSGetTimeoutTriggerTime(GHSConnectionHandle conHandle, double *time)
{
    uint16_t requestID;
    const char *rpcRequest;
    GHSReturnValue returnVal;
    JsonRpcNamedParameter *outParams = NULL;
    size_t nOutParams = 0;
    size_t i;

    if (!time)
        return GHSReturnValue_NullPtrArgument;

    requestID = ConnectionNextRequestID(&conHandle);
    rpcRequest = JsonRpcCreateRequest("GetTimeoutTriggerTime", requestID, NULL, 0);
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *time = 0.0;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "Time") == 0)
            *time = outParams[i].value.dValue;
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSSetTimeoutTriggerTime(GHSConnectionHandle conHandle, double time)
{
    JsonRpcNamedParameter inParams[1];
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest;

    inParams[0] = JsonRpcCreateDoubleNamedParameter("Time", time);
    rpcRequest = JsonRpcCreateRequest("SetTimeoutTriggerTime", requestID, inParams, NELEMENTS(inParams));
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSGetContinuousRecordingMode(GHSConnectionHandle conHandle, GHSSlotID slotID, GHSContinuousRecordingMode *continuousMode)
{
   char slotid[2];
    JsonRpcNamedParameter inParams[1];
    uint16_t requestID;
    const char *rpcRequest;
    GHSReturnValue returnVal;
    JsonRpcNamedParameter *outParams = NULL;
    size_t nOutParams = 0;
    size_t i;

    if (!continuousMode)
        return GHSReturnValue_NullPtrArgument;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    requestID = ConnectionNextRequestID(&conHandle);
    rpcRequest = JsonRpcCreateRequest("GetContinuousRecordingMode", requestID, inParams, NELEMENTS(inParams));
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *continuousMode = GHSContinuousRecordingMode_Reserved;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "ContinuousRecordingMode") == 0)
            *continuousMode = (GHSContinuousRecordingMode)outParams[i].value.dValue;
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSSetContinuousRecordingMode(GHSConnectionHandle conHandle, GHSSlotID slotID, GHSContinuousRecordingMode continuousMode)
{
    char slotid[2];
    JsonRpcNamedParameter inParams[2];
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest;

    if (GHSContinuousRecordingMode_Standard != continuousMode && GHSContinuousRecordingMode_Circular != continuousMode
       && GHSContinuousRecordingMode_Limited != continuousMode && GHSContinuousRecordingMode_StopOnTrigger != continuousMode)
    {
        return GHSReturnValue_InvalidContinuousMode;
    }

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    inParams[1] = JsonRpcCreateDoubleNamedParameter("ContinuousRecordingMode", continuousMode);
    rpcRequest = JsonRpcCreateRequest("SetContinuousRecordingMode", requestID, inParams, NELEMENTS(inParams));
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSGetContinuousTimeSpan(GHSConnectionHandle conHandle, GHSSlotID slotID, double *timeSpan)
{
    char slotid[2];
    JsonRpcNamedParameter inParams[1];
    uint16_t requestID;
    const char *rpcRequest;
    GHSReturnValue returnVal;
    JsonRpcNamedParameter *outParams = NULL;
    size_t nOutParams = 0;
    size_t i;

    if (!timeSpan)
        return GHSReturnValue_NullPtrArgument;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    requestID = ConnectionNextRequestID(&conHandle);
    rpcRequest = JsonRpcCreateRequest("GetContinuousTimeSpan", requestID, inParams, NELEMENTS(inParams));
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *timeSpan = 0.0;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "ContinuousTimeSpan") == 0)
            *timeSpan = outParams[i].value.dValue;
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSSetContinuousTimeSpan(GHSConnectionHandle conHandle, GHSSlotID slotID, double timeSpan)
{
    char slotid[2];
    JsonRpcNamedParameter inParams[2];
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    inParams[1] = JsonRpcCreateDoubleNamedParameter("ContinuousTimeSpan", timeSpan);
    rpcRequest = JsonRpcCreateRequest("SetContinuousTimeSpan", requestID, inParams, NELEMENTS(inParams));
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSGetContinuousLeadOutTime(GHSConnectionHandle conHandle, GHSSlotID slotID,  double *leadOutTime)
{
    char slotid[2];
    JsonRpcNamedParameter inParams[1];
    uint16_t requestID;
    const char *rpcRequest;
    GHSReturnValue returnVal;
    JsonRpcNamedParameter *outParams = NULL;
    size_t nOutParams = 0;
    size_t i;

    if (!leadOutTime)
        return GHSReturnValue_NullPtrArgument;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    requestID = ConnectionNextRequestID(&conHandle);
    rpcRequest = JsonRpcCreateRequest("GetContinuousLeadOutTime", requestID, inParams, NELEMENTS(inParams));
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *leadOutTime = 0.0;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "ContinuousLeadOutTime") == 0)
            *leadOutTime = outParams[i].value.dValue;
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSSetContinuousLeadOutTime(GHSConnectionHandle conHandle, GHSSlotID slotID,  double leadOutTime)
{
    char slotid[2];
    JsonRpcNamedParameter inParams[2];
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest;

    slotid[0] = slotID;
    slotid[1] = '\0';
    inParams[0] = JsonRpcCreateStringNamedParameter("SlotId", slotid);
    inParams[1] = JsonRpcCreateDoubleNamedParameter("ContinuousLeadOutTime", leadOutTime);
    rpcRequest = JsonRpcCreateRequest("SetContinuousLeadOutTime", requestID, inParams, NELEMENTS(inParams));
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

GHSReturnValue GHSInitiateFieldBusDataTransfer(GHSConnectionHandle conHandle, int *updateRate, int *dataCount)
{
    JsonRpcNamedParameter inParams[1];
    uint16_t requestID;
    const char *rpcRequest;
    GHSReturnValue returnVal;
    JsonRpcNamedParameter *outParams = NULL;
    size_t nOutParams = 0;
    size_t i;
    unsigned short fieldBusPort = 0;
    size_t bufferDepth = 0;
    int nClientAllocatedDataSize;

    if (!updateRate || !dataCount)
        return GHSReturnValue_NullPtrArgument;

    nClientAllocatedDataSize = *dataCount;

    inParams[0] = JsonRpcCreateIntNamedParameter("UpdateRate", *updateRate);
    requestID = ConnectionNextRequestID(&conHandle);
    rpcRequest = JsonRpcCreateRequest("InitiateFieldBusDataTransfer", requestID, inParams, NELEMENTS(inParams));
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "FieldBusPort") == 0)
            fieldBusPort = (unsigned short)outParams[i].value.dValue;
        if (strcmp(outParams[i].key, "BufferDepth") == 0)
            bufferDepth = (size_t)outParams[i].value.dValue;
        if (strcmp(outParams[i].key, "UpdateRate") == 0)
            *updateRate = (int)outParams[i].value.dValue;
        if (strcmp(outParams[i].key, "DataCount") == 0)
        {
            *dataCount = (int)outParams[i].value.dValue;
            if (nClientAllocatedDataSize < *dataCount)
                returnVal = GHSReturnValue_FieldBusError_BufferSizeInvalid;
        }
    }

    FreeJsonParams(outParams, nOutParams);
    if (returnVal != GHSReturnValue_OK)
        return returnVal;

    g_GenDaqFieldBusHandle->dataHandle = fieldBus_buf_Init(bufferDepth, *dataCount);
    if (g_GenDaqFieldBusHandle->dataHandle == NULL)
        return GHSReturnValue_FieldBusError_NullFieldBusRingBuffer;

    g_GenDaqFieldBusHandle->LastPacket.fValues = (float*)malloc(sizeof(float) * (*dataCount));

    /* Connection and creation of the FieldBus Handle */
    return ConnectionEstablish(&g_GenDaqFieldBusHandle->conHandle, g_GenDaqFieldBusHandle->ipAddress, fieldBusPort);
}

GHSReturnValue GHSStopFieldBusDataTransfer(GHSConnectionHandle conHandle)
{
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest = JsonRpcCreateRequest("StopFieldBusDataTransfer", requestID, NULL, 0);
    GHSReturnValue returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
    if (returnVal != GHSReturnValue_OK)
        return returnVal;

    if (!g_GenDaqFieldBusHandle->dataHandle || !g_GenDaqFieldBusHandle->LastPacket.fValues || !g_GenDaqFieldBusHandle->ipAddress || !g_GenDaqFieldBusHandle->conHandle)
        return GHSReturnValue_NullPtrArgument;

    (void)ConnectionClose(&g_GenDaqFieldBusHandle->conHandle);

    free(g_GenDaqFieldBusHandle->LastPacket.fValues);
    fieldBus_buf_Free(g_GenDaqFieldBusHandle->dataHandle);
    g_GenDaqFieldBusHandle->dataHandle = 0;

    return GHSReturnValue_OK;
}

GHSReturnValue GHSGetFieldBusDataCount(GHSConnectionHandle conHandle, int *dataCount)
{
    uint16_t requestID;
    const char *rpcRequest;
    GHSReturnValue returnVal;
    JsonRpcNamedParameter *outParams = NULL;
    size_t nOutParams = 0;
    size_t i;

    if (!dataCount)
        return GHSReturnValue_NullPtrArgument;

    requestID = ConnectionNextRequestID(&conHandle);
    rpcRequest = JsonRpcCreateRequest("GetFieldBusDataCount", requestID, NULL, 0);
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *dataCount = 0;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "DataCount") == 0)
            *dataCount = (int)outParams[i].value.dValue;
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSGetFieldBusDataNameAndUnit(GHSConnectionHandle conHandle, int dataIndex, char **dataName, char **dataUnit)
{
    uint16_t requestID;
    const char *rpcRequest;
    GHSReturnValue returnVal;
    JsonRpcNamedParameter *outParams = NULL;
    JsonRpcNamedParameter inParams[1];
    size_t nOutParams = 0;
    size_t i;

    if (!dataName || !dataUnit)
        return GHSReturnValue_NullPtrArgument;

    inParams[0] = JsonRpcCreateIntNamedParameter("DataIndex", dataIndex);

    requestID = ConnectionNextRequestID(&conHandle);
    rpcRequest = JsonRpcCreateRequest("GetFieldBusDataNameAndUnit", requestID, inParams, NELEMENTS(inParams));

    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *dataName = 0;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "DataName") == 0)
            *dataName = stringDuplicate(outParams[i].value.sValue);
        if (strcmp(outParams[i].key, "DataUnit") == 0)
            *dataUnit = stringDuplicate(outParams[i].value.sValue);
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSReadNextSnapshot(GHSConnectionHandle conHandle, GHSFunctionCallType callType, double *timestamp, float *values, int *overrun)
{
    (void)conHandle; /* Only one field bus connection is supported for now */

    if (!g_GenDaqFieldBusHandle->dataHandle)
        return GHSReturnValue_FieldBusError_NullFieldBusRingBuffer;

    if (!fieldBus_buf_GetSize(g_GenDaqFieldBusHandle->dataHandle) && callType == GHS_NonBlockingCall)
    {
        *timestamp = g_GenDaqFieldBusHandle->LastPacket.dTimestamp;
        memcpy(values, g_GenDaqFieldBusHandle->LastPacket.fValues, sizeof(float) * g_GenDaqFieldBusHandle->dataHandle->dataCount);
        *overrun = 0;
    }
    else
    {
        *overrun = fieldBus_buf_GetAsyncData(g_GenDaqFieldBusHandle->dataHandle, timestamp, values, callType);
        if (*overrun)
            g_GenDaqFieldBusHandle->dataHandle->overrun = 0;
    }

    return GHSReturnValue_OK;
}

GHSReturnValue GHSReceiveFieldBusData(GHSConnectionHandle conHandle)
{
    FieldBusData packet;
    GHSReturnValue returnVal;
    int packetSize;

    (void)conHandle; /* Only one field bus connection is supported for now */

    if(!g_GenDaqFieldBusHandle->dataHandle || !g_GenDaqFieldBusHandle->conHandle)
        return GHSReturnValue_FieldBusError_NotConnected;

    packetSize = g_GenDaqFieldBusHandle->dataHandle->dataCount;
    packet.fValues = (float*) malloc(sizeof(float) * packetSize);

    returnVal = FieldBusConnectionRead(&g_GenDaqFieldBusHandle->conHandle, &packet, packetSize);
    if (returnVal != GHSReturnValue_OK)
    {
        free(packet.fValues);
        return returnVal;
    }

    fieldBus_buf_PutAsyncData(g_GenDaqFieldBusHandle->dataHandle, packet.dTimestamp, packet.fValues);

    g_GenDaqFieldBusHandle->LastPacket.dTimestamp = packet.dTimestamp;

    memcpy(g_GenDaqFieldBusHandle->LastPacket.fValues, packet.fValues, sizeof(float) * packetSize);

    free(packet.fValues);
    return returnVal;
}

GHSReturnValue GHSRequestFieldBusSnapshot(GHSConnectionHandle conHandle, double *timestamp, int *dataCount, float *values)
{
    uint16_t requestID;
    const char *rpcRequest;
    GHSReturnValue returnVal;
    JsonRpcNamedParameter *outParams = NULL;
    size_t nOutParams = 0;
    size_t i;
    int nClientAllocatedDataSize;

    if (!timestamp || !dataCount || !values)
        return GHSReturnValue_NullPtrArgument;

    nClientAllocatedDataSize = *dataCount;

    requestID = ConnectionNextRequestID(&conHandle);
    rpcRequest = JsonRpcCreateRequest("RequestSnapshot", requestID, NULL, 0);
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *timestamp = 0;
    *dataCount = 0;
    *values = 0;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "TimeStamp") == 0)
            *timestamp = outParams[i].value.dValue;
        if (strcmp(outParams[i].key, "DataCount") == 0)
        {
            *dataCount = (int)outParams[i].value.dValue;
            if (nClientAllocatedDataSize < *dataCount)
                returnVal = GHSReturnValue_FieldBusError_BufferSizeInvalid;
        }
        if (strcmp(outParams[i].key, "Data") == 0)
        {
            int count = MIN(*dataCount, nClientAllocatedDataSize);
            int k;

            for (k = 0; k < count; k++)
                values[k] = (float) outParams[i].value.arValue[k];
        }
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSGetCANAcqControl(GHSConnectionHandle conHandle, GHSEnableDisable *enabled, int *busID, int *msgID)
{
    uint16_t requestID;
    const char *rpcRequest;
    GHSReturnValue returnVal;
    JsonRpcNamedParameter *outParams = NULL;
    size_t nOutParams = 0;
    size_t i;

    if (!enabled || !msgID || !busID)
        return GHSReturnValue_NullPtrArgument;

    requestID = ConnectionNextRequestID(&conHandle);
    rpcRequest = JsonRpcCreateRequest("GetCANAcqControl", requestID, NULL, 0);
    returnVal = ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, &outParams, &nOutParams);

    /* Default value, in case the outputs are not found in our response */
    *enabled = GHS_Disable;
    *msgID = 0;
    *busID = 0;

    for (i = 0; i < nOutParams; i++)
    {
        if (strcmp(outParams[i].key, "Enable") == 0)
            *enabled = (GHSEnableDisable)outParams[i].value.dValue;
        if (strcmp(outParams[i].key, "MsgID") == 0)
            *msgID = (int)outParams[i].value.dValue;
        if (strcmp(outParams[i].key, "BusID") == 0)
            *busID = (int)outParams[i].value.dValue;
    }

    FreeJsonParams(outParams, nOutParams);
    return returnVal;
}

GHSReturnValue GHSSetCANAcqControl(GHSConnectionHandle conHandle, GHSEnableDisable enabled, int busID, int msgID)
{
    JsonRpcNamedParameter inParams[3];
    uint16_t requestID = ConnectionNextRequestID(&conHandle);
    const char *rpcRequest;

    inParams[0] = JsonRpcCreateIntNamedParameter("Enable", enabled);
    inParams[1] = JsonRpcCreateIntNamedParameter("BusID", busID);
    inParams[2] = JsonRpcCreateIntNamedParameter("MsgID", msgID);
    rpcRequest = JsonRpcCreateRequest("SetCANAcqControl", requestID, inParams, NELEMENTS(inParams));
    return ConnectionSendRequestWaitResponse(&conHandle, rpcRequest, requestID, NULL, NULL);
}

static void FreeJsonParams(JsonRpcNamedParameter *params, size_t numParams)
{
    size_t i;

    if (!params)
        return;

    for (i = 0; i < numParams; i++)
    {
        if (params[i].vType == VT_string)
            free((void *)params[i].value.sValue);

        free((void *)params[i].key);
    }

    free(params);
}

char *stringDuplicate(const char *input)
{
    int length = 0;
    char *output = NULL;

    /* Determine length */
    length = (int)strlen(input) + 1;
    /* Allocate memory for duplicate string */
    output = malloc(length);

    if (output)
        memcpy (output, input, length);

    return output;
}
