/*
 * Copyright (C) 2009 HBM Netherlands B.V.
 * Schutweg 15a
 * 5145 NP Waalwijk
 * The Netherlands
 * http://www.hbm.com
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

/**
 * @file GEN_DAQ_API.h
 * @author HBM Netherlands B.V.
 * @date 25 September 2018
 * @brief Header file for the GEN DAQ API
 *
 * All functions supported by this API are present in this header-file and in this file only.
 *
 * ******************************************************************************
 * HBM is committed to keep this API stable and downwards compatible as long as
 * this is technically feasible. HBM reserves the right to change the internal
 * implementation and therefore no rights can be derived from the implementation.
 * ******************************************************************************
 */

#ifndef GEN_DAQ_API_H
#define GEN_DAQ_API_H

#include <stddef.h>
#include <stdint.h>

#if defined(__cplusplus)
extern "C" { /* Make sure we have C-declarations in C++ programs */
#endif

    /**
     * @brief A unique identifier per mainframe connection.
     *
     * It is used by all API calls to distinguish between mainframes.
     */
    typedef uint32_t GHSConnectionHandle;

    /**
     * @brief Pointer to a blob containing all mainframe settings.
     *
     * The internal structure of this blob is undefined.
     */
    typedef void* GHSSettingsPtr;

    /**
     * @brief Identifier of the slot used to address recorders. Consists of a capital letter (e.g. 'A').
     */
    typedef char GHSSlotID;

    /**
     * @brief All possible API return values.
     *
     * This enumeration will be extended and/or changed as the API grows.
     */
    typedef enum GHSReturnValue
    {
        /** @brief Reserved to distinguish zero initialized GHSReturnValues. Should never be returned. */
        GHSReturnValue_Reserved = 0,
        /** @brief Function call succeeded. */
        GHSReturnValue_OK = 1,
        /** @brief Undefined error occurred. */
        GHSReturnValue_NOK = 2,
        /** @brief The slot referred to does not contain a recorder. */
        GHSReturnValue_EmptySlot = 3,
        /** @brief Passed pointer is a null pointer. */
        GHSReturnValue_NullPtrArgument = 4,
        /** @brief The slot referred to does not exist on this mainframe. */
        GHSReturnValue_InvalidSlotID = 5,
        /** @brief Preview or recording can not be started because the system is not idle. */
        GHSReturnValue_SystemNotIdle = 6,
        /** @brief Recording can not be paused or stopped because the system is not recording. FieldBus is also not available */
        GHSReturnValue_SystemNotRecording = 7,
        /** @brief Recording can not be resumed because the system is not paused. */
        GHSReturnValue_SystemNotPaused = 8,
        /** @brief Sample-rate is not supported by this recorder. */
        GHSReturnValue_InvalidSampleRate = 9,
        /** @brief The connection-handle does not refer to a valid connected mainframe. */
        GHSReturnValue_InvalidHandle = 10,
        /** @brief The client API (with this header file) is incompatible with the version on the server. */
        GHSReturnValue_APIMismatch = 11,
        /** @brief A connection could not be established with the provided IP and Port number. */
        GHSReturnValue_ConnectionFailed = 12,
        /** @brief The ipAddress provided is not formatted properly. ("127.0.0.1") */
        GHSReturnValue_InvalidIP = 13,
        /** @brief The API call did not complete in time on the mainframe. */
        GHSReturnValue_MainframeTimeout = 14,
        /** @brief The mainframe disk is full. */
        GHSReturnValue_InsufficientDiskSpace = 15,
        /** @brief The mainframe could not create a recording. */
        GHSReturnValue_CreateRecordingFailed = 16,
        /** @brief Connection was broken or client not logged on. */
        GHSReturnValue_NoConnection = 17,
        /** @brief The mainframe is not configured for local storage. */
        GHSReturnValue_IncompatibleStorageLocation = 18,
        /** @brief The indicated recording could not be found. */
        GHSReturnValue_RecordingNotFound = 19,
        /** @brief Preview can not be stopped because the system is not in preview. */
        GHSReturnValue_SystemNotInPreview = 20,
        /** @brief A connection could not be established because there was already a connection. */
        GHSReturnValue_AlreadyConnected = 21,
        /** @brief The recording name was not valid. */
        GHSReturnValue_InvalidRecordingName = 22,
        /** @brief The channel referred to does not exist on this recorder. */
        GHSReturnValue_InvalidChannelIndex = 23,
        /** @brief The user mode reffered to does not exist. */
        GHSReturnValue_InvalidUserMode = 24,
        /** @brief The channel specified has a type that is not supported for this method. */
        GHSReturnValue_InvalidChannelType = 25,
        /** @brief The trigger position percentage is not valid. */
        GHSReturnValue_InvalidTriggerPosition = 26,
        /** @brief The sweep mode referred to does not exist. */
        GHSReturnValue_InvalidSweepMode = 27,
        /** @brief There are no recorders in the mainframe. */
        GHSReturnValue_NoRecordersInMainframe = 28,
        /** @brief The continuous recording mode referred to does not exist. */
        GHSReturnValue_InvalidContinuousMode = 29,
        /** @brief The current sweep mode does not support setting the trigger position. */
        GHSReturnValue_InvalidModeForTriggerPosition = 30,
        /** @brief The setter method succeeded but the value has been adapted. */
        GHSReturnValue_Adapted = 31,
        /** @brief An invalid UTF-8 character was found. */
        GHSReturnValue_InvalidUTF8Character = 32,
        /** @brief Duplicate channel name within a recorder. */
        GHSReturnValue_DuplicateChannelName = 33,
        /** @brief Unsupported data type in the method. */
        GHSReturnValue_InvalidDataType = 34,
        /** @brief Method is not defined in the Mainframe side */
        GHSReturnValue_MethodNotFound = 35,
        /** @brief Invalid JSON Format received in the Mainframe side */
        GHSReturnValue_InvalidJSONFormat = 36,
        /** @brief Undefined Error Message */
        GHSReturnValue_UnkownErrorMessage = 37,
        /** @brief FieldBusError: Field bus is not configured to publish any data */
        GHSReturnValue_FieldBusError_NotConfigured = 38,
        /** @brief FieldBusError: Requesting snapshot not available when fieldbus is already enabled */
        GHSReturnValue_FieldBusError_FieldBusAlready_Enabled = 39,
        /** @brief FieldBusError: FieldBusRing Buffer handle is null */
        GHSReturnValue_FieldBusError_NullFieldBusRingBuffer = 40,
        /** @brief FieldBusError: FieldBus Packet is of an invalid size */
        GHSReturnValue_FieldBusError_InvalidFieldBusPacket = 41,
        /** @brief FieldBusError: FieldBus is not connected */
        GHSReturnValue_FieldBusError_NotConnected = 42,
        /** @brief There are no formulas deployed for FieldBus(excluding timestamp) */
        GHSReturnValue_FieldBusError_NoFormulasDeployed = 43,
        /** @brief The size for the allocated buffer is not enough */
        GHSReturnValue_FieldBusError_BufferSizeInvalid = 44,
        /** @brief Field bus is not supported in this mainframe */
        GHSReturnValue_FieldBusError_FieldBusNotSupported = 45,
        /** @brief The specified CAN bus could not be found */
        GHSReturnValue_CANBusNotFound = 46,
        /** @brief Error for Write access */
        GHSReturnValue_WriteAccessBlocked = 47,
        /** @brief The specified CAN message ID is invalid */
        GHSReturnValue_InvalidCANMsgID = 48
    } GHSReturnValue;

    /**
     * @brief Acquisition states.
     */
    typedef enum GHSAcquisitionState
    {
        /** @brief Reserved to distinguish zero initialized GHSAcquisitionState. Should never be returned. */
        GHSAcquisitionState_Reserved = 0,
        /** @brief Mainframe is recording. */
        GHSAcquisitionState_Recording = 1,
        /** @brief Mainframe is in pause state. */
        GHSAcquisitionState_Pause = 2,
        /** @brief Mainframe stopped recording but data is not saved yet. */
        GHSAcquisitionState_SavingData = 3,
        /** @brief Mainframe is idle and ready to be put into other states. */
        GHSAcquisitionState_Idle = 4,
        /** @brief Mainframe is in preview state. */
        GHSAcquisitionState_Preview = 5
    } GHSAcquisitionState;

    /**
     * @brief Master timebase synchronization status values.
     */
    typedef enum GHSSyncStatus
    {
        /** @brief Reserved to distinguish zero initialized GHSSyncStatus. Should never be returned. */
        GHSSyncStatus_Reserved = 0,
        /** @brief Not yet in sync with external signal: both master.*/
        GHSSyncStatus_NotSynced = 1,
        /** @brief PLL is busy locking the phase of the master. */
        GHSSyncStatus_Syncing = 2,
        /** @brief PLL phase is locked and accurate. */
        GHSSyncStatus_Synced = 3,
        /** @brief PLL phase was lost and a restart is not allowed now. */
        GHSSyncStatus_ReSyncing = 4,
        /** @brief No external 1PPS pulse was received for some time. */
        GHSSyncStatus_NoSignal = 5,
        /** @brief PLL is coarse synced. */
        GHSSyncStatus_CoarseSynced = 6,
        /** @brief We are not synced to GMR1000 PTP master.*/
        GHSSyncStatus_NoGMR1000 = 7,
        /** @brief  We are not synced to a OTMC100 PTP master. */
        GHSSyncStatus_NoOTMC100 = 8
    } GHSSyncStatus;

    /**
     * @brief Enable/Disable values used in various methods.
     */
    typedef enum GHSEnableDisable
    {
        /** @brief Disable. */
        GHS_Disable = 0,
        /** @brief Enable. */
        GHS_Enable = 1
    } GHSEnableDisable;

    /**
     * @brief Physical channel types.
     */
    typedef enum GHSChannelType
    {
        /** @brief Reserved to distinguish zero initialized GHSChannelType. Should never be returned. */
        GHSChannelType_Invalid = 0,
        /** @brief Analog Channel. */
        GHSChannelType_Analog = 1,
        /** @brief Event Channel. */
        GHSChannelType_Event = 2,
        /** @brief Timer Counter Channel. */
        GHSChannelType_TimerCounter = 3
    } GHSChannelType;

    /**
     * @brief Storage locations.
     */
    typedef enum GHSStorageLocation
    {
        /** @brief Reserved to distinguish zero initialized GHSStorageLocation. Should never be returned. */
        GHSStorageLocation_Reserved = 0,
        /** @brief PC storage. */
        GHSStorageLocation_Remote = 1,
        /** @brief Storage on local disk #1 (like SCSI or SATA). */
        GHSStorageLocation_Local1 = 2,
        /** @brief Storage on local disk #2 (like SCSI or SATA). */
        GHSStorageLocation_Local2 = 3
    } GHSStorageLocation;

    /**
     * @brief User modes.
     */
    typedef enum GHSUserMode
    {
        /** @brief Reserved to distinguish zero initialized GHSUserMode. Should never be returned. */
        GHSUserMode_Reserved = 0,
        /** @brief Acquire a single, triggered event. */
        GHSUserMode_Sweeps = 1,
        /** @brief Acquire a continuous stream of data. */
        GHSUserMode_Continuous  = 2,
        /** @brief Acquire a continuous stream of data with triggered events. */
        GHSUserMode_Dual  = 3
    } GHSUserMode;

    /**
     * @brief Trigger modes.
     */
    typedef enum GHSTriggerMode
    {
        /** @brief No trigger mode. */
        GHSTriggerMode_Off = 0,
        /** @brief Basic trigger mode. */
        GHSTriggerMode_Basic = 1,
        /** @brief Dual trigger mode. */
        GHSTriggerMode_Dual = 2,
        /** @brief Window trigger mode. */
        GHSTriggerMode_Window = 3,
        /** @brief Dual window trigger mode. */
        GHSTriggerMode_DualWindow = 4,
        /** @brief Sequential trigger mode. */
        GHSTriggerMode_Sequential = 5,
        /** @brief Use primary trigger level as qualifier. */
        GHSTriggerMode_QualifierBasic = 6,
        /** @brief Use both trigger levels as qualifiers. */
        GHSTriggerMode_QualifierDual = 7
    } GHSTriggerMode;

    /**
     * @brief Directions of level crossing detection.
     */
    typedef enum GHSDirection
    {
        /** @brief Rising edge level crossing detection direction. */
        GHSDirection_RisingEdge = 0,
        /** @brief Falling edge level crossing detection direction. */
        GHSDirection_FallingEdge = 1
    } GHSDirection;

    /**
     * @brief Signal coupling values.
     */
    typedef enum GHSSignalCoupling
    {
        /** @brief Ground the input. */
        GHSSigCpl_GND = 0,
        /** @brief Measure directly. */
        GHSSigCpl_DC = 1,
        /** @brief Measure AC component. */
        GHSSigCpl_AC = 2,
        /** @brief Measure RMS without offset correction. */
        GHSSigCpl_DC_RMS = 3,
        /** @brief Measure RMS of AC component. */
        GHSSigCpl_AC_RMS = 4,
        /** @brief */
        GHSSigCpl_DC_Frequency = 5,
        /** @brief */
        GHSSigCpl_AC_Frequency = 6,
        /** @brief */
        GHSSigCpl_DC_TrueRMS = 7,
        /** @brief */
        GHSSigCpl_AC_TrueRMS = 8,
        /** @brief */
        GHSSigCpl_DC_ExternalProbe = 9,
        /** @brief */
        GHSSigCpl_AC_ExternalProbe = 10,
        /** @brief Reference voltage or reference signal. */
        GHSSigCpl_Reference = 11,
        /** @brief Quadrature counter reset. */
        GHSSigCpl_ZeroReset = 12,
        /** @brief Quadradure single precision. */
        GHSSigCpl_SinglePrecision = 13,
        /** @brief Quadradure double precision. */
        GHSSigCpl_DoublePrecision = 14,
        /** @brief Quadradure precision. */
        GHSSigCpl_QuadPrecision = 15,
        /** @brief */
        GHSSigCpl_Charge = 16
    } GHSSignalCoupling;

    /**
     * @brief Input coupling values that define what signals from the input BNCs are passed on to the amplifier.
     */
    typedef enum GHSInputCoupling
    {
        /** @brief Measure the voltage on the +input. */
        GHSInCpl_SingleEndedPositive = 0,
        /** @brief Measure the voltage on the -input. */
        GHSInCpl_SingleEndedNegative = 1,
        /** @brief Measure the difference between +input and -input. */
        GHSInCpl_Differential = 2,
        /** @brief Measure the current flowing from +input to -input. */
        GHSInCpl_Current = 3,
        /** @brief Measure the difference between +input and -input without having a connection to ground. */
        GHSInCpl_FloatingDifferential = 4
    } GHSInputCoupling;

    /**
     * @brief Filter types.
     */
    typedef enum GHSFilterType
    {
        /** @brief Bessel filter. */
        GHSFilterType_Bessel = 0,
        /** @brief Butterworth filter. */
        GHSFilterType_Butterworth = 1,
        /** @brief Elliptic filter. */
        GHSFilterType_Elliptic = 2,
        /** @brief FIR filter.*/
        GHSFilterType_FIR = 3,
        /** @brief IIR filter. */
        GHSFilterType_IIR = 4,
        /** @brief Wideband filter. */
        GHSFilterType_Wideband = 5,
        /** @brief Bessel AA filter. */
        GHSFilterType_Bessel_AA = 6,
        /** @brief Butterworth AA filter. */
        GHSFilterType_Butterworth_AA = 7,
        /** @brief Sigma delta wide band filter. */
        GHSFilterType_SigmaDeltaWB = 8,
        /** @brief Sigma delta filter. */
        GHSFilterType_SigmaDelta = 9,
        /** @brief Band pass filter. */
        GHSFilterType_BandPass = 10,
        /** @brief FIR 3dB filter. */
        GHSFilterType_FIR3dB = 11
    } GHSFilterType;

    /**
     * @brief Amplifier modes.
     */
    typedef enum GHSAmplifierMode
    {
        /** @brief No amplifier mode. Should never be returned. */
        GHSAmplifierMode_None = -1,
        /** @brief Differential voltage amplifier mode. */
        GHSAmplifierMode_Basic = 0,
        /** @brief Bridge amplifier mode. */
        GHSAmplifierMode_Bridge = 1,
        /** @brief Accelerometer (ICP) amplifier mode. */
        GHSAmplifierMode_Icp = 2,
        /** @brief Thermo couple amplifier mode. */
        GHSAmplifierMode_ThermoCouple = 3,
        /** @brief Differential voltage amplifier mode with excitation. */
        GHSAmplifierMode_BasicSensor = 4,
        /** @brief Charge amplifier mode. */
        GHSAmplifierMode_Charge = 5,
        /** @brief 4..20mA current amplifier mode. */
        GHSAmplifierMode_Current4_20 = 6,
        /** @brief Resistance type temperature sensor amplifier mode. */
        GHSAmplifierMode_ThermoResistor = 7
    } GHSAmplifierMode;

    /**
     * @brief Excitation types
     */
    typedef enum GHSExcitationType
    {
        /** @brief Voltage excitation type. */
        GHSExcitationType_Voltage = 0,
        /** @brief Voltage sense excitation type. */
        GHSExcitationType_Voltage_Sense = 1,
        /** @brief Current excitation type. */
        GHSExcitationType_Current = 2,
        /** @brief Voltage strobed excitation type. */
        GHSExcitationType_Voltage_Strobed = 3,
        /** @brief Voltage sense stobed excitation type. */
        GHSExcitationType_Voltage_Sense_Strobed = 4,
        /** @brief Current strobed excitation type. */
        GHSExcitationType_Current_Strobed = 5
    } GHSExcitationType;

    /**
     * @brief Sweep recording modes.
     */
    typedef enum GHSSweepRecordingMode
    {
        /** @brief Reserved to distinguish zero initialized GHSSweepRecordingMode. Should never be returned. */
        GHSSweepMode_Reserved = 0,
        /** @brief Normal sweep recording mode. */
        GHSSweepMode_Normal = 1,
        /** @brief Pre-trigger sweep recording mode. */
        GHSSweepMode_PreTrigger = 2
    } GHSSweepRecordingMode;

    /**
     * @brief Continuous recording modes.
     */
    typedef enum GHSContinuousRecordingMode
    {
        /** @brief Reserved to distinguish zero initialized GHSContinuousRecordingMode. Should never be returned. */
        GHSContinuousRecordingMode_Reserved = 0,
        /** @brief Continuous acquisition without a specific stop condition. */
        GHSContinuousRecordingMode_Standard = 1,
        /** @brief Data is acquired in a circular buffer of specified length. */
        GHSContinuousRecordingMode_Circular = 2,
        /** @brief Stop after specified total time is acquired. */
        GHSContinuousRecordingMode_Limited = 3,
        /** @brief Stop when a trigger occurs. */
        GHSContinuousRecordingMode_StopOnTrigger = 4
    } GHSContinuousRecordingMode;

    /**
     * @brief Timer/counter channel modes.
     */
    typedef enum GHSTimerCounterMode
    {
        /** @brief Uni-directional RPM timer/counter channel mode. */
        GHSTimerCounterMode_RPMUniDirectional = 0,
        /** @brief Bi-directional RPM timer/counter channel mode. */
        GHSTimerCounterMode_RPMBiDirectional = 1,
        /** @brief Quadrature RPM timer/counter channel mode. */
        GHSTimerCounterMode_RPMQuadrature = 2,
        /** @brief Uni-directional frequency timer/counter channel mode. */
        GHSTimerCounterMode_FrequencyUniDirectional = 3,
        /** @brief Bi-directional frequency timer/counter channel mode. */
        GHSTimerCounterMode_FrequencyBiDirectional = 4,
        /** @brief Quadrature frequency timer/counter channel mode. */
        GHSTimerCounterMode_FrequencyQuadrature = 5,
        /** @brief Uni-directional count timer/counter channel mode. */
        GHSTimerCounterMode_CountUniDirectional = 6,
        /** @brief Bi-directional count timer/counter channel mode. */
        GHSTimerCounterMode_CountBiDirectional = 7,
        /** @brief Quadrature count timer/counter channel mode. */
        GHSTimerCounterMode_CountQuadrature = 8,
        /** @brief Quadrature angle timer/counter channel mode. */
        GHSTimerCounterMode_AngleQuadrature = 9,
        /** @brief Quadrature angle timer/counter channel mode with reference position. */
        GHSTimerCounterMode_AngleQuadratureWithRefPos = 10,
        /** @brief Uni-directional angle timer/counter channel mode. */
        GHSTimerCounterMode_AngleUniDirectional = 11,
        /** @brief Uni-directional angle timer/counter channel mode with reference position. */
        GHSTimerCounterMode_AngleUniDirectionalWithRefPos = 12,
        /** @brief Bi-directional angle timer/counter channel mode. */
        GHSTimerCounterMode_AngleBiDirectional = 13,
        /** @brief Bi-directional angle timer/counter channel mode with reference position. */
        GHSTimerCounterMode_AngleBiDirectionalWithRefPos = 14
    } GHSTimerCounterMode;

    /**
     * @brief Function call type.
     */
    typedef enum GHSFunctionCallType
    {
        /** @brief The function call doe not block execution. */
        GHS_NonBlockingCall = 0,
        /** @brief The function call blocks execution. */
        GHS_BlockingCall = 1
    } GHSFunctionCallType;

    /**
     * @brief Recording data source.
     */
    typedef enum GHSRecordingDataSource
    {
        /** @brief Synchronous recorder channels. */
        GHSRecordingDataSource_SyncChannels = 0,
        /** @brief Synchronous real-time formulas. */
        GHSRecordingDataSource_SyncRealTimeFormulas = 1
    } GHSRecordingDataSource;

    /**
     * @brief Trigger arm state.
     */
    typedef enum GHSTriggerArmState
    {
        /** @brief Triggers are disarmed. */
        GHSTriggerArmState_Disarmed = 0,
        /** @brief Triggers are armed. */
        GHSTriggerArmState_Armed = 1
    } GHSTriggerArmState;

    /**
     * @brief Sweep trigger mode.
     */
    typedef enum GHSSweepTriggerMode
    {
        /** @brief Create a sweep recording for the first trigger only. */
        GHSSweepTriggerMode_FirstTrigger = 0,
        /** @brief Create a sweep recording for all triggers. */
        GHSSweepTriggerMode_EveryTrigger = 1,
        /** @brief Create a sweep recording on a stop trigger. */
        GHSSweepTriggerMode_StopTrigger = 2
    } GHSSweepTriggerMode;

    /**
     * @brief Client access (permission).
     */
    typedef enum GHSAccess
    {
        /** @brief Client has read-only access. */
        GHSAccess_ReadOnly = 0,
        /** @brief Client has read and write access. */
        GHSAccess_ReadWrite = 1
    } GHSAccess;

    /**
     * @defgroup Connections Connections
     * @brief Connection related API functions.
     */

    /**
     * @brief Establishes a connection to the mainframe.
     *
     * @param [in] ipAddress IP address needs to be an IPV4 address. "localhost" is also supported.
     * @param [in] portno TCP port number (currently defined as 8006).
     * @param [out] conHandle An unique identifier per mainframe connection @ref GHSConnectionHandle.
     *                        This parameter is used by all other API calls to distinguish between mainframes.
     * @param [out] serverAPIVersion The API version returned by the server.
     * @return @ref GHSReturnValue
     *
     * @ingroup Connections
     * @sa @ref GHSDisconnect()
     */
    GHSReturnValue GHSConnect(const char *ipAddress, unsigned short portno, GHSConnectionHandle *conHandle, uint32_t *serverAPIVersion);

    /**
     * @brief Disconnects from a connected mainframe.
     *
     * Results in error when the handle is not valid and / or mainframe is not connected.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @return @ref GHSReturnValue
     *
     * @ingroup Connections
     * @sa @ref GHSConnect()
     */
    GHSReturnValue GHSDisconnect(GHSConnectionHandle conHandle);

    /**
     * @brief Returns the API version number of this client.
     *
     * This function can be useful when @ref GHSConnect() returns GHSReturnValue_APIMismatch.
     *
     * @return version number
     *
     * @ingroup Connections
     * @sa @ref GHSConnect()
     */
    uint32_t GHSGetClientAPIVersion(void);

    /**
     * @brief Returns the current access permissions for this client.
     *
     * The current access permissions might change at any time.
     * For example, when Perception connects concurrently to the same mainframe, write access is dropped for the connected GEN DAQ API client.
     * If a concurrently connected Perception disconnects, write access is granted to the connected GEN DAQ API client.
     * API functions that require write access will return @ref GHSReturnValue_WriteAccessBlocked in case this client currently has no write access.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [out] access The current access @ref GHSAccess.
     * @return @ref GHSReturnValue
     *
     * @ingroup Connections
     */
    GHSReturnValue GHSGetCurrentAccess(GHSConnectionHandle conHandle, GHSAccess *access);

    /**
     * @defgroup AcquisitionControl Acquisition control
     * @brief Acquisition control related API functions.
     */

    /**
     * @brief Starts preview mode.
     *
     * The system needs to be idle before calling this function.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @return @ref GHSReturnValue
     *
     * @ingroup AcquisitionControl
     * @sa @ref Page4_Sec1, @ref GHSStopPreview()
     */
    GHSReturnValue GHSStartPreview(GHSConnectionHandle conHandle);

    /**
     * @brief Stops preview mode.
     *
     * The system needs to be in preview mode before calling this function.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @return @ref GHSReturnValue
     *
     * @ingroup AcquisitionControl
     * @sa @ref Page4_Sec1, @ref GHSStartPreview()
     */
    GHSReturnValue GHSStopPreview(GHSConnectionHandle conHandle);

    /**
     * @brief Starts a recording on local storage.
     *
     * The system needs to be idle before calling this function.
     * Note that the connected mainframe will generate a recording name.
     * This command can be executed only successfully when the local storage is set.
     *
     * @param [in] conHandle        The unique connection identifier returned by @ref GHSConnect().
     * @return @ref GHSReturnValue
     *
     * @ingroup AcquisitionControl
     * @sa @ref Page4_Sec1, @ref GHSStopRecording(), @ref GHSPauseRecording()
     */
    GHSReturnValue GHSStartRecording(GHSConnectionHandle conHandle);

    /**
     * @brief Pauses a started recording.
     *
     * The system needs to be recording before calling this function.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @return @ref GHSReturnValue
     *
     * @ingroup AcquisitionControl
     * @sa @ref Page4_Sec1, @ref GHSStopRecording(), @ref GHSResumeRecording()
     */
    GHSReturnValue GHSPauseRecording(GHSConnectionHandle conHandle);

    /**
     * @brief Resumes a paused recording.
     *
     * The system needs to be paused before calling this function.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @return @ref GHSReturnValue
     *
     * @ingroup AcquisitionControl
     * @sa @ref Page4_Sec1, @ref GHSPauseRecording()
     */
    GHSReturnValue GHSResumeRecording(GHSConnectionHandle conHandle);

    /**
     * @brief Stops a started recording.
     *
     * The system needs to be recording before calling this function.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @return @ref GHSReturnValue
     *
     * @ingroup AcquisitionControl
     * @sa @ref Page4_Sec1, @ref GHSStartRecording(), @ref GHSPauseRecording()
     */
    GHSReturnValue GHSStopRecording(GHSConnectionHandle conHandle);

    /**
     * @brief Issues a trigger.
     *
     * Results in error when the trigger cannot be issued.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @return @ref GHSReturnValue
     *
     * @ingroup AcquisitionControl
     */
    GHSReturnValue GHSTrigger(GHSConnectionHandle conHandle);

    /**
     * @brief Returns the Acquisition State of the Mainframe
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [out] state The acquisition state @ref GHSAcquisitionState.
     * @return @ref GHSReturnValue
     *
     * @ingroup AcquisitionControl
     */
    GHSReturnValue GHSGetAcquisitionState(GHSConnectionHandle conHandle, GHSAcquisitionState *state);

    /**
     * @brief Retrieves the current acquisition time relative to the start of acquistion.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [out] acquisitionTime The acquisition time in seconds.
     * @return @ref GHSReturnValue
     *
     * @ingroup AcquisitionControl
     * @sa @ref GHSGetAcquisitionStartTime()
     */
    GHSReturnValue GHSGetAcquisitionTime(GHSConnectionHandle conHandle, double *acquisitionTime);

    /**
     * @brief Retrieves the absolute time of the start of acquisition.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [out] year The year
     * @param [out] day The day number in the year
     * @param [out] seconds The number of seconds since midnight.
     * @return @ref GHSReturnValue
     *
     * @ingroup AcquisitionControl
     * @sa @ref GHSGetAcquisitionTime()
     */
    GHSReturnValue GHSGetAcquisitionStartTime(GHSConnectionHandle conHandle, int *year, int *day, double *seconds);

    /**
     * @defgroup Mainframe Mainframe
     * @brief Mainframe related API functions.
     */

    /**
     * @brief Enable or disable the identification sound of the connected mainframe.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] enabled The identify enabled status @ref GHSEnableDisable.
     * @return @ref GHSReturnValue
     *
     * @ingroup Mainframe
     */
    GHSReturnValue GHSIdentify(GHSConnectionHandle conHandle, GHSEnableDisable enabled);

    /**
     * @brief Get total and available mainframe internal disk space.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [out] total Total mainframe internal disk space in GB.
     * @param [out] available Available internal disk space in GB
     * @return @ref GHSReturnValue
     *
     * @ingroup Mainframe
     */
    GHSReturnValue GHSGetDiskSpace(GHSConnectionHandle conHandle, double *total, double *available);

    /**
     * @brief Determine the mainframe sync status.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [out] syncStatus Sync status @ref GHSSyncStatus.
     * @return @ref GHSReturnValue
     *
     * @ingroup Mainframe
     */
    GHSReturnValue GHSGetSyncStatus(GHSConnectionHandle conHandle, GHSSyncStatus *syncStatus);

    /**
     * @brief Retrieve the number of slots in the mainframe.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [out] slotCount The number of slots in the mainframe.
     * @return @ref GHSReturnValue
     *
     * @ingroup Mainframe
     */
    GHSReturnValue GHSGetSlotCount(GHSConnectionHandle conHandle, int *slotCount);

    /**
     * @brief Retrieve the user mode.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [out] userMode The user mode @ref GHSUserMode.
     * @return @ref GHSReturnValue
     *
     * @ingroup Mainframe
     * @sa @ref GHSSetUserMode()
     */
    GHSReturnValue GHSGetUserMode(GHSConnectionHandle conHandle, GHSUserMode *userMode);

    /**
     * @brief Set the user mode.
     * 
     * @attention The system needs to be idle before calling this function.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] userMode The desired user mode @ref GHSUserMode.
     * @return @ref GHSReturnValue
     *
     * @ingroup Mainframe
     * @sa @ref GHSGetUserMode()
     */
    GHSReturnValue GHSSetUserMode(GHSConnectionHandle conHandle, GHSUserMode userMode);

    /**
     * @brief Determine type, name, serial number and firmware version information for the connected mainframe.
     *
     * @attention The mainframeType, mainframeName, serialNumber and firmwareVersion parameters are UTF-8 encoded.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [out] mainframeType The type of the mainframe.
     * @param [out] mainframeName The name of the mainframe.
     * @param [out] serialNumber The serial number of the mainframe.
     * @param [out] firmwareVersion The firmware version of the mainframe.
     * @return @ref GHSReturnValue
     *
     * @ingroup Mainframe
     */
    GHSReturnValue GHSGetMainframeInformation(GHSConnectionHandle conHandle, char **mainframeType, char **mainframeName, char **serialNumber, char **firmwareVersion);

    /**
     * @defgroup ManageRecordings Manage recordings
     * @brief Recording management related API functions.
     */

    /**
     * @brief Retrieve the storage location.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [out] storageLocation The storage location @ref GHSStorageLocation.
     * @return @ref GHSReturnValue
     *
     * @ingroup ManageRecordings
     * @sa @ref GHSSetStorageLocation()
     */
    GHSReturnValue GHSGetStorageLocation(GHSConnectionHandle conHandle, GHSStorageLocation *storageLocation);

    /**
     * @brief Set the storage location.
     * 
     * @attention The system needs to be idle before calling this function.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] storageLocation The desired storage location @ref GHSStorageLocation.
     * @return @ref GHSReturnValue
     *
     * @ingroup ManageRecordings
     * @sa @ref GHSGetStorageLocation()
     */
    GHSReturnValue GHSSetStorageLocation(GHSConnectionHandle conHandle, GHSStorageLocation storageLocation);

    /**
     * @brief Retrieve the recording base name and recording index of the last recording file.
     *
     * @attention The recording base name parameter is UTF-8 encoded.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [out] recordingBaseName The base name of the recording file.
     * @param [out] recordingIndex The index of the recording file.
     * @return @ref GHSReturnValue
     *
     * @ingroup ManageRecordings
     * @sa @ref GHSSetRecordingName()
     */
    GHSReturnValue GHSGetRecordingName(GHSConnectionHandle conHandle, const char **recordingBaseName, int *recordingIndex);

    /**
     * @brief Set the recording base name and recording index for the next recording file.
     *
     * @attention The system needs to be idle before calling this function.\n
     * The recording base name parameter must be UTF-8 encoded.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] recordingBaseName The desired base name of the recording file.
     * @param [in] recordingIndex The desired index of the recording file.
     * @return @ref GHSReturnValue
     *
     * @ingroup ManageRecordings
     * @sa @ref GHSGetRecordingName()
     */
    GHSReturnValue GHSSetRecordingName(GHSConnectionHandle conHandle, const char *recordingBaseName, int recordingIndex);

    /**
     * @brief Sets the recording base name.
     *
     * @deprecated This function is deprecated and is scheduled to be removed from future versions of this API.\n
     * Please use @ref GHSSetRecordingName() instead.
     *
     * @attention The system needs to be idle before calling this function.\n
     * The recording base name parameter must be UTF-8 encoded.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] recordingBaseName The base name of the recording file.
     * @return @ref GHSReturnValue
     *
     * @ingroup ManageRecordings
     */
    GHSReturnValue GHSSetRecordingBaseName(GHSConnectionHandle conHandle, const char *recordingBaseName);

    /**
     * @brief Deletes the most recent recording from local mainframe storage.
     *
     * @attention Recordings are deleted asynchronously.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @return @ref GHSReturnValue
     *
     * @ingroup ManageRecordings
     * @sa @ref GHSDeleteAllRecordings()
     */
    GHSReturnValue GHSDeleteLastRecording(GHSConnectionHandle conHandle);

    /**
     * @brief Deletes all recordings from local mainframe storage.
     *
     * @attention Recordings are deleted asynchronously.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @return @ref GHSReturnValue
     *
     * @ingroup ManageRecordings
     * @sa @ref GHSDeleteLastRecording()
     */
    GHSReturnValue GHSDeleteAllRecordings(GHSConnectionHandle conHandle);

    /**
     * @brief Retrieve storage enabled status of high and low rate data for the specified recording data source.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] source The recording data source to retrieve the storage enabled status for.
     * @param [in] slotID The slot containing the recorder to retrieve the storage enabled status for (e.g. 'A' for the first slot).
     *             This argument is only evaluated for recorder based data sources. See @ref GHSRecordingDataSource.
     * @param [out] highRateEnabled Flag to indicate if high rate data is stored.
     * @param [out] lowRateEnabled Flag to indicate if low rate data is stored.
     * @return @ref GHSReturnValue
     *
     * @ingroup ManageRecordings
     * @sa @ref GHSSetHighLowRateStorageEnabled()
     */
    GHSReturnValue GHSGetHighLowRateStorageEnabled(GHSConnectionHandle conHandle, GHSRecordingDataSource source, GHSSlotID slotID, GHSEnableDisable *highRateEnabled, GHSEnableDisable *lowRateEnabled);

    /**
     * @brief Enable/disable storage of high and low rate data for the specified recording data source.
     * 
     * @attention The system needs to be idle before calling this function.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] source The recording data source to enable/disable data storage for.
     * @param [in] slotID The slot containing the recorder to enable/disable data storage for (e.g. 'A' for the first slot).
     *             This argument is only evaluated for recorder based data sources. See @ref GHSRecordingDataSource.
     * @param [in] highRateEnabled Enable/disable storage of high rate data.
     * @param [in] lowRateEnabled Enable/disable storage of low rate data.
     * @return @ref GHSReturnValue
     *
     * @ingroup ManageRecordings
     * @sa @ref GHSGetHighLowRateStorageEnabled()
     */
    GHSReturnValue GHSSetHighLowRateStorageEnabled(GHSConnectionHandle conHandle, GHSRecordingDataSource source, GHSSlotID slotID, GHSEnableDisable highRateEnabled, GHSEnableDisable lowRateEnabled);

    /**
     * @defgroup ManageSettings Manage mainframe settings
     * @brief Settings management related API functions.
     */

    /**
     * @brief Retrieves the current mainframe settings as a blob.
     *
     * As this blob can be of variable size, it is upon the caller to ensure reading the correct amount of memory.\n
     * Memory for this blob is allocated by the API.\n
     * To free these resources a call to @ref GHSFreeSettings() needs to be made.\n
     *
     * @image html Settings_SaveGENSettingsToDisk.png
     * @image latex Settings_SaveGENSettingsToDisk.png
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [out] blob Pointer to the settings blob.
     * @param [out] blobSize Size of the settings blob.
     * @return @ref GHSReturnValue
     *
     * @ingroup ManageSettings
     * @sa @ref GHSSetCurrentSettings(), @ref GHSFreeSettings()
     */
    GHSReturnValue GHSGetCurrentSettings(GHSConnectionHandle conHandle, GHSSettingsPtr *blob, size_t *blobSize);

    /**
     * @brief Applies the mainframe settings contained in the input argument.
     *
     * Note that this function overwrites any previously set settings and / or applied setup.
     *
     * @image html Settings_LoadGENSettingsFromDisk.png
     * @image latex Settings_LoadGENSettingsFromDisk.png
     *
     * @attention The system needs to be idle before calling this function.
     * 
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] blob Pointer to the settings blob.
     * @param [in] blobSize Size of the settings blob.
     * @return @ref GHSReturnValue
     *
     * @ingroup ManageSettings
     * @sa @ref GHSGetCurrentSettings()
     */
    GHSReturnValue GHSSetCurrentSettings(GHSConnectionHandle conHandle, GHSSettingsPtr blob, size_t blobSize);

    /**
     * @brief Frees the resources allocated for the settings blob.
     *
     * @param [in] blob Pointer to the settings blob as returned by the @ref GHSGetCurrentSettings() API function.
     * @return @ref GHSReturnValue
     *
     * @ingroup ManageSettings
     * @sa @ref GHSGetCurrentSettings()
     */
    GHSReturnValue GHSFreeSettings(GHSSettingsPtr blob);

    /**
     * @brief Persists the current mainframe settings.
     *
     * @attention The persisted mainframe settings are applied upon a mainframe boot.
     *
     * @image html Settings_CopyActiveSettingsToBoot.png
     * @image latex Settings_CopyActiveSettingsToBoot.png
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @return @ref GHSReturnValue
     *
     * @ingroup ManageSettings
     * @sa @ref GHSApplyPersistedSettings()
     */
    GHSReturnValue GHSPersistCurrentSettings(GHSConnectionHandle conHandle);

    /**
     * @brief A mainframe might contain persisted settings (being applied upon boot). This method re-applies these settings.
     *
     * In Perception this maps on the 'Configured boot' feature.\n
     *
     * @attention The system needs to be idle before calling this function.\n
     * This function overwrites any previously set settings and / or persisted settings.
     *
     * @image html Settings_CopyBootToActiveSettings.png
     * @image latex Settings_CopyBootToActiveSettings.png
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @return @ref GHSReturnValue
     *
     * @ingroup ManageSettings
     * @sa @ref GHSPersistCurrentSettings()
     */
    GHSReturnValue GHSApplyPersistedSettings(GHSConnectionHandle conHandle);

    /**
     * @defgroup Recorder Recorder
     * @brief Recorder related API functions.
     */

    /**
     * @brief Determine type, name, serial number and firmware version information for a recorder.
     *
     * @attention The recorderType, recorderName, serialNumber and firmwareVersion parameters are UTF-8 encoded.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder from which to retrieve the recorder information (e.g. 'A' for the first slot).
     * @param [out] recorderType The type of the recorder.
     * @param [out] recorderName The name of the recorder.
     * @param [out] serialNumber The serial number of the recorder.
     * @param [out] firmwareVersion The firmware version of the recorder.
     * @return @ref GHSReturnValue
     *
     * @ingroup Recorder
     * @sa @ref GHSSetContinuousLeadOutTime()
     */
    GHSReturnValue GHSGetRecorderInformation(GHSConnectionHandle conHandle, GHSSlotID slotID, char **recorderType, char **recorderName, char **serialNumber, char **firmwareVersion);

    /**
     * @brief Determine if recorder is enabled or disabled.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder from which to retrieve the enabled status (e.g. 'A' for the first slot).
     * @param [out] enabled The recorder enabled status @ref GHSEnableDisable.
     * @return @ref GHSReturnValue
     *
     * @ingroup Recorder
     * @sa @ref GHSSetRecorderEnabled()
     */
    GHSReturnValue GHSGetRecorderEnabled(GHSConnectionHandle conHandle, GHSSlotID slotID, GHSEnableDisable *enabled);

    /**
     * @brief Enable or disable a recorder.
     * 
     * @attention The system needs to be idle before calling this function.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder to enable or disable (e.g. 'A' for the first slot).
     * @param [in] enabled Set to GHS_Enable/GHS_Disable to enable/disable the recorder @ref GHSEnableDisable.
     * @return @ref GHSReturnValue
     *
     * @ingroup Recorder
     * @sa @ref GHSGetRecorderEnabled()
     */
    GHSReturnValue GHSSetRecorderEnabled(GHSConnectionHandle conHandle, GHSSlotID slotID, GHSEnableDisable enabled);

    /**
     * @brief Retrieve the number of channels for a recorder.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder to get number of channels for (e.g. 'A' for the first slot).
     * @param [out] channelCount The number of channels for the recorder.
     * @return @ref GHSReturnValue
     *
     * @ingroup Recorder
     */
    GHSReturnValue GHSGetChannelCount(GHSConnectionHandle conHandle, GHSSlotID slotID, int *channelCount);

    /**
     * @brief Determine the sample rate for a recorder.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder from which to retrieve the sample rate (e.g. 'A' for the first slot).
     * @param [out] sampleRate in samples per second.
     * @return @ref GHSReturnValue
     *
     * @ingroup Recorder
     * @sa @ref GHSSetSampleRate()
     */
    GHSReturnValue GHSGetSampleRate(GHSConnectionHandle conHandle, GHSSlotID slotID, double *sampleRate);

    /**
     * @brief Set the sample rate for a recorder.
     *
     * @attention The system needs to be idle before calling this function.\n
     * This function overwrites any previously set sample rate setting for the specified recorder.\n
     * If the specified sample rate is not supported by the recorder, the sample rate is rounded to the nearest supported sample rate.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder for which to set the sample rate (e.g. 'A' for the first slot).
     * @param [in] sampleRate in samples per second.
     * @return @ref GHSReturnValue
     *
     * @ingroup Recorder
     * @sa @ref GHSGetSampleRate()
     */
    GHSReturnValue GHSSetSampleRate(GHSConnectionHandle conHandle, GHSSlotID slotID, double sampleRate);

    /**
     * @defgroup Channel Channel
     * @brief Channel related API functions.
     */

    /**
     * @brief Determine the type of a channel.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder that the channel belongs to (e.g. 'A' for the first slot).
     * @param [in] channelIndex The zero-based index of the channel to determine the type for.
     * @param [out] channelType Type of the channel @ref GHSChannelType.
     * @return @ref GHSReturnValue
     *
     * @ingroup Channel
     */
    GHSReturnValue GHSGetChannelType(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, GHSChannelType *channelType);

    /**
     * @brief Determine the name of a channel.
     *
     * @attention The channelName parameter is UTF-8 encoded.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder that the channel belongs to (e.g. 'A' for the first slot).
     * @param [in] channelIndex The zero-based index of the channel to determine the name of.
     * @param [out] channelName The name of the channel.
     * @return @ref GHSReturnValue
     *
     * @ingroup Channel
     * @sa @ref GHSSetChannelName()
     */
    GHSReturnValue GHSGetChannelName(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, char **channelName);

    /**
     * @brief Set the name for a channel.
     *
     * @attention The channelName parameter must be UTF-8 encoded.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder that the channel belongs to (e.g. 'A' for the first slot).
     * @param [in] channelIndex The zero-based index of the channel to set the name for.
     * @param [in] channelName The desired channel name.
     * @return @ref GHSReturnValue
     *
     * @ingroup Channel
     * @sa @ref GHSGetChannelName()
     */
    GHSReturnValue GHSSetChannelName(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, const char *channelName);

    /**
     * @brief Determine if storage is enabled or disabled for a channel.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder that the channel belongs to (e.g. 'A' for the first slot).
     * @param [in] channelIndex The zero-based index of the channel to determine the storage status for.
     * @param [out] enabled The storage enabled status for the channel @ref GHSEnableDisable.
     * @return @ref GHSReturnValue
     *
     * @ingroup Channel
     * @sa @ref GHSSetChannelStorageEnabled()
     */
    GHSReturnValue GHSGetChannelStorageEnabled(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, GHSEnableDisable *enabled);

    /**
     * @brief Enable or disable storage for a channel.
     *
     * @attention The system needs to be idle before calling this function.
     * 
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot of the recorder that channel belongs to (e.g. 'A' for the first slot).
     * @param [in] channelIndex The zero-based index of the channel to set the storage status for.
     * @param [in] enabled The desired storage enabled status for the channel @ref GHSEnableDisable.
     * @return @ref GHSReturnValue
     *
     * @ingroup Channel
     * @sa @ref GHSGetChannelStorageEnabled()
     */
    GHSReturnValue GHSSetChannelStorageEnabled(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, GHSEnableDisable enabled);

    /**
     * @defgroup AnalogChannel Analog
     * @brief Analog channel related API functions.
     *
     * @ingroup Channel
     */

    /**
     * @brief Determine the trigger settings for an analog channel.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder that the analog channel belongs to (e.g. 'A' for the first slot).
     * @param [in] channelIndex The zero-based index of the analog channel to determine the trigger settings for.
     * @param [out] triggerMode Trigger Mode @ref GHSTriggerMode.
     * @param [out] primaryLevel The primary trigger level.
     * @param [out] secondaryLevel The secondary trigger level.
     * @param [out] hysteresis The trigger hysteresis.
     * @param [out] direction The trigger direction @ref GHSDirection.
     * @return @ref GHSReturnValue
     *
     * @ingroup AnalogChannel
     * @sa @ref GHSSetTriggerSettings()
     */
    GHSReturnValue GHSGetTriggerSettings(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, GHSTriggerMode *triggerMode, double *primaryLevel, double *secondaryLevel, double *hysteresis, GHSDirection *direction);

    /**
     * @brief Set the trigger settings for an analog channel.
     *
     * @attention The system needs to be idle before calling this function.\n
     * This function overwrites any previously set trigger settings for the specified recorder.\n
     * If the specified trigger mode or value is not supported by the recorder, the trigger mode remains unchanged or the value is rounded to the nearest supported value.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder that the analog channel belongs to (e.g. 'A' for the first slot).
     * @param [in] channelIndex The zero-based index of the analog channel to set the trigger settings for.
     * @param [in] triggerMode Trigger Mode @ref GHSTriggerMode. Default is TriggerMode_Basic.
     * @param [in] primaryLevel The desired primary trigger level.
     * @param [in] secondaryLevel The desired secondary trigger level.
     * @param [in] hysteresis The desired  trigger hysteresis.
     * @param [in] direction The desired trigger direction @ref GHSDirection.
     * @return @ref GHSReturnValue
     *
     * @ingroup AnalogChannel
     * @sa @ref GHSGetTriggerSettings()
     */
    GHSReturnValue GHSSetTriggerSettings(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, GHSTriggerMode triggerMode, double primaryLevel, double secondaryLevel, double hysteresis, GHSDirection direction);

    /**
     * @brief Determine the signal coupling for an analog channel.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder that the analog channel belongs to (e.g. 'A' for the first slot).
     * @param [in] channelIndex The zero-based index of the analog channel to determine the signal coupling for.
     * @param [out] signalCoupling The signal coupling @ref GHSSignalCoupling.
     * @return @ref GHSReturnValue
     *
     * @ingroup AnalogChannel
     * @sa @ref GHSSetSignalCoupling()
     */
    GHSReturnValue GHSGetSignalCoupling(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, GHSSignalCoupling *signalCoupling);

    /**
     * @brief Set the signal coupling for an analog channel.
     *
     * @attention If the specified signal coupling mode is not supported by the recorder, the signal coupling mode remains unchanged.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder that the analog channel belongs to (e.g. 'A' for the first slot).
     * @param [in] channelIndex The zero-based index of the analog channel to set the signal coupling for.
     * @param [in] signalCoupling The desired signal coupling @ref GHSSignalCoupling.
     * @return @ref GHSReturnValue
     *
     * @ingroup AnalogChannel
     * @sa @ref GHSGetSignalCoupling()
     */
    GHSReturnValue GHSSetSignalCoupling(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, GHSSignalCoupling signalCoupling);

    /**
     * @brief Determine the input coupling for an analog channel.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder that the analog channel belongs to (e.g. 'A' for the first slot).
     * @param [in] channelIndex The zero-based index of the analog channel to determine the input coupling for.
     * @param [out] inputCoupling The input coupling @ref GHSInputCoupling.
     * @return @ref GHSReturnValue
     *
     * @ingroup AnalogChannel
     * @sa @ref GHSSetInputCoupling()
     */
    GHSReturnValue GHSGetInputCoupling(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, GHSInputCoupling *inputCoupling);

    /**
     * @brief Set the input coupling for an analog channel.
     *
     * @attention The system needs to be idle before calling this function.\n
     * If the specified input coupling mode is not supported by the recorder, the input coupling mode remains unchanged.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder that the analog channel belongs to (e.g. 'A' for the first slot).
     * @param [in] channelIndex The zero-based index of the analog channel to set the input coupling for.
     * @param [in] inputCoupling The desired input coupling @ref GHSInputCoupling.
     * @return @ref GHSReturnValue
     *
     * @ingroup AnalogChannel
     * @sa @ref GHSSetInputCoupling()
     */
    GHSReturnValue GHSSetInputCoupling(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, GHSInputCoupling inputCoupling);

    /**
     * @brief Determine the span and offset for an analog channel.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder that the analog channel belongs to (e.g. 'A' for the first slot).
     * @param [in] channelIndex The zero-based index of the analog channel to determine the span and offset for.
     * @param [out] span The span in user units.
     * @param [out] offset The offset in user units.
     * @return @ref GHSReturnValue
     *
     * @ingroup AnalogChannel
     * @sa @ref GHSSetSpanAndOffset()
     */
    GHSReturnValue GHSGetSpanAndOffset(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, double *span, double *offset);

    /**
     * @brief Set Span and offset for analog channels
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder that the analog channel belongs to (e.g. 'A' for the first slot).
     * @param [in] channelIndex The zero-based index of the analog channel to set the span and offset for.
     * @param [in] span The span in user units. The value is adapted to available options.
     * @param [in] offset The offset in user units. The value is adapted to available options.
     * @return @ref GHSReturnValue
     *
     * @ingroup AnalogChannel
     * @sa @ref GHSGetSpanAndOffset()
     */
    GHSReturnValue GHSSetSpanAndOffset(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, double span, double offset);

    /**
     * @brief Determine the filter type and frequency for an analog channel.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder that the analog channel belongs to (e.g. 'A' for the first slot).
     * @param [in] channelIndex The zero-based index of the analog channel to determine the filter type and frequency for.
     * @param [out] filterType The filter type @ref GHSFilterType.
     * @param [out] frequency The filter frequency in Hz.
     * @return @ref GHSReturnValue
     *
     * @ingroup AnalogChannel
     * @sa @ref GHSSetFilterTypeAndFrequency()
     */
    GHSReturnValue GHSGetFilterTypeAndFrequency(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, GHSFilterType *filterType, double *frequency);

    /**
     * @brief Set the filter type and frequency for an analog channel.
     *
     * @attention This function overwrites any previously set filter settings for the specified recorder.\n
     * If a specified filter type or value is not supported by the recorder, the filter type remains the same or the value is rounded to a supported vale.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder that the analog channel belongs to (e.g. 'A' for the first slot).
     * @param [in] channelIndex The zero-based index of the analog channel to set the filter type and frequency for.
     * @param [in] filterType The filter type @ref GHSFilterType. Default is GHSFilterType_Bessel.
     * @param [in] frequency The filter frequency in Hz.
     * @return @ref GHSReturnValue
     *
     * @ingroup AnalogChannel
     * @sa @ref GHSGetFilterTypeAndFrequency()
     */
    GHSReturnValue GHSSetFilterTypeAndFrequency(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, GHSFilterType filterType, double frequency);

    /**
     * @brief Determine the excitation type and value for an analog channel.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder that the channel belongs to (e.g. 'A' for the first slot).
     * @param [in] channelIndex The zero-based index of the channel to determine the excitation type for.
     * @param [out] excitationType The excitation type @ref GHSExcitationType.
     * @param [out] excitationValue The excitation value in user units (voltage or current).
     * @return @ref GHSReturnValue
     *
     * @ingroup AnalogChannel
     * @sa @ref GHSSetExcitation()
     */
    GHSReturnValue GHSGetExcitation(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, GHSExcitationType *excitationType, double *excitationValue);

    /**
     * @brief Set the excitation type and value for an analog channel.
     *
     * @attention The system needs to be idle before calling this function.\n
     * If the specified excitation type or value is not supported by the recorder,
     * the excitation type remains unchanged or the value is rounded to the nearest supported value.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder that the channel belongs to (e.g. 'A' for the first slot).
     * @param [in] channelIndex The zero-based index of the channel to set the excitation type for.
     * @param [in] excitationType The desired excitation type @ref GHSExcitationType. Default is ExcitationType_Voltage.
     * @param [in] excitationValue The desired excitation value in user units (voltage or current).
     * @return @ref GHSReturnValue
     *
     * @ingroup AnalogChannel
     * @sa @ref GHSGetExcitation()
     */
    GHSReturnValue GHSSetExcitation(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, GHSExcitationType excitationType, double excitationValue);

    /**
     * @brief Determine the amplifier mode for an analog channel.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder that the channel belongs to (e.g. 'A' for the first slot).
     * @param [in] channelIndex The zero-based index of the channel to determine the amplifier mode for.
     * @param [out] amplifierMode The amplifier mode @ref GHSAmplifierMode.
     * @return @ref GHSReturnValue
     *
     * @ingroup AnalogChannel
     * @sa @ref GHSSetAmplifierMode()
     */
    GHSReturnValue GHSGetAmplifierMode(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, GHSAmplifierMode *amplifierMode);

    /**
     * @brief Set the amplifier mode for an analog channel.
     *
     * @attention The system needs to be idle before calling this function.\n
     * If the specified amplifier mode is not supported by the recorder, the amplifier mode remains unchanged.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder that the channel belongs to (e.g. 'A' for the first slot).
     * @param [in] channelIndex The zero-based index of the channel to set the amplifier mode for.
     * @param [in] amplifierMode The desired amplifier mode @ref GHSAmplifierMode.
     * @return @ref GHSReturnValue
     *
     * @ingroup AnalogChannel
     * @sa @ref GHSGetAmplifierMode()
     */
    GHSReturnValue GHSSetAmplifierMode(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, GHSAmplifierMode amplifierMode);

    /**
     * @brief Determine the technical units, unit multiplier and unit offset for an analog channel.
     *
     * @attention The units parameter is UTF-8 encoded.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder that the analog channel belongs to (e.g. 'A' for the first slot).
     * @param [in] channelIndex The zero-based index of the analog channel to determine the technical units for.
     * @param [out] units The technical units (e.g. 'V' for Volt or 'Hz' for Hertz).
     * @param [out] multiplier The technical units multiplier value.
     * @param [out] offset The technical units offset value.
     * @return @ref GHSReturnValue
     *
     * @ingroup AnalogChannel
     * @sa @ref GHSSetTechnicalUnits()
     */
    GHSReturnValue GHSGetTechnicalUnits(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, char **units, double *multiplier, double *offset);

    /**
     * @brief Set the technical units, unit multiplier and unit offset for an analog channel.
     *
     * @attention The units parameter must be UTF-8 encoded.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder that the analog channel belongs to (e.g. 'A' for the first slot).
     * @param [in] channelIndex The zero-based index of the analog channel to set the technical units for.
     * @param [in] units The desired technical units (e.g. 'V' for Volt or 'Hz' for Hertz).
     * @param [in] multiplier The desired technical units multiplier value.
     * @param [in] offset The desired technical units offset value.
     * @return @ref GHSReturnValue
     *
     * @ingroup AnalogChannel
     * @sa @ref GHSGetTechnicalUnits()
     */
    GHSReturnValue GHSSetTechnicalUnits(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, const char *units, double multiplier, double offset);

    /**
     * @brief Determine the auto range enable and time settings.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder that the analog channel belongs to (e.g. 'A' for the first slot).
     * @param [in] channelIndex The zero-based index of the analog channel to determine the span and offset for.
     * @param [out] autoRangeEnabled The auto range enabled setting.
     * @param [out] autoRangeTime The time for auto range in seconds.
     * @return @ref GHSReturnValue
     *
     * @ingroup AnalogChannel
     * @sa @ref GHSSetAutoRange()
     */
    GHSReturnValue GHSGetAutoRange(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, GHSEnableDisable *autoRangeEnabled, double *autoRangeTime);

    /**
     * @brief Set Auto range settings for analog channels
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder that the analog channel belongs to (e.g. 'A' for the first slot).
     * @param [in] channelIndex The zero-based index of the analog channel to set the span and offset for.
     * @param [in] autoRangeEnabled The auto range enabled setting. The value is adapted to available options.
     * @param [in] autoRangeTime The time for auto range in seconds. The value is adapted to available options.
     * @return @ref GHSReturnValue
     *
     * @ingroup AnalogChannel
     * @sa @ref GHSGetAutoRange()
     */
    GHSReturnValue GHSSetAutoRange(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, GHSEnableDisable autoRangeEnabled, double autoRangeTime);

    /**
     * @brief Command a single shot for auto range
     * 
     * @attention The system needs to be acquiring for this function to have any effect.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder that the analog channel belongs to (e.g. 'A' for the first slot).
     * @param [in] channelIndex The zero-based index of the analog channel to set the span and offset for.
     * @param [in] autoRangeTime The time for auto range in seconds. The value is adapted to available options.
     * @return @ref GHSReturnValue
     *
     * @ingroup AnalogChannel
     */
    GHSReturnValue GHSCmdAutoRangeNow(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, double autoRangeTime);

    /**
     * @brief Retrieve calibration information for an analog channel.
     *
     * @attention The calibrationDateTime, verificationDateTime, powerVerificationDateTime, calibrationLab, verificationLab and powerVerificationLab parameters are UTF-8 encoded.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder that the analog channel belongs to (e.g. 'A' for the first slot).
     * @param [in] channelIndex The zero-based index of the analog channel to retrieve calibration information for.
     * @param [out] calibrationDateTime The date and time this analog channel has been calibrated.
     * @param [out] verificationDateTime The date and time the calibration for this analog channel has been verified.
     * @param [out] powerVerificationDateTime The date and time the power calibration for this analog channel has been verified (if applicable).
     * @param [out] calibrationLab The laboratory that conducted the calibration for this analog channel.
     * @param [out] verificationLab The laboratory that verified the calibration for this analog channel.
     * @param [out] powerVerificationLab The laboratory that verified the power calibration for this analog channel (if applicable).
     * @return @ref GHSReturnValue
     *
     * @ingroup AnalogChannel
     */
    GHSReturnValue GHSGetChannelCalibrationInformation(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, char** calibrationDateTime, char** verificationDateTime, char** powerVerificationDateTime, char** calibrationLab, char** verificationLab, char** powerVerificationLab);

    /**
     * @defgroup TimerCounterChannel Timer/Counter
     * @brief Timer/Counter channel related API functions.
     *
     * @ingroup Channel
     */

    /**
     * @brief Determine the mode for a timer/counter channel.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder that the timer/counter channel belongs to (e.g. 'A' for the first slot).
     * @param [in] channelIndex The zero-based index of the timer/counter channel to determine the mode for.
     * @param [out] mode The timer/counter mode @ref GHSTimerCounterMode.
     * @return @ref GHSReturnValue
     *
     * @ingroup TimerCounterChannel
     * @sa @ref GHSSetTimerCounterMode()
     */
    GHSReturnValue GHSGetTimerCounterMode(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, GHSTimerCounterMode *mode);

    /**
     * @brief Set the mode for a timer/counter channel.
     *
     * @attention The system needs to be idle before calling this function.\n
     * If the specified timer/counter mode is not supported by the recorder, the timer/counter mode remains unchanged.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder that the timer/counter channel belongs to (e.g. 'A' for the first slot).
     * @param [in] channelIndex The zero-based index of the timer/counter channel to set the mode for.
     * @param [in] mode The desired timer/counter mode @ref GHSTimerCounterMode. Default is TimerCounterMode_RPMUniDirectional.
     * @return @ref GHSReturnValue
     *
     * @ingroup TimerCounterChannel
     * @sa @ref GHSGetTimerCounterMode()
     */
    GHSReturnValue GHSSetTimerCounterMode(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, GHSTimerCounterMode mode);

    /**
     * @brief Determine the gate time for a timer/counter channel.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder that the timer/counter channel belongs to (e.g. 'A' for the first slot).
     * @param [in] channelIndex The zero-based index of the timer/counter channel to determine the gate time for.
     * @param [out] gateTime The gate time in seconds.
     * @return @ref GHSReturnValue
     *
     * @ingroup TimerCounterChannel
     * @sa @ref GHSSetTimerCounterGateTime()
     */
    GHSReturnValue GHSGetTimerCounterGateTime(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, double *gateTime);

    /**
     * @brief Set the gate time for a timer/counter channel.
     *
     * @attention The system needs to be idle before calling this function.\n
     * If the specified timer/counter gate time is not supported by the recorder, the timer/counter gate time is rounded to the nearest supported gate time.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder that the timer/counter channel belongs to (e.g. 'A' for the first slot).
     * @param [in] channelIndex The zero-based index of the timer/counter channel to set the gate time for.
     * @param [in] gateTime The desired gate time in seconds.
     * @return @ref GHSReturnValue
     *
     * @ingroup TimerCounterChannel
     * @sa @ref GHSGetTimerCounterGateTime()
     */
    GHSReturnValue GHSSetTimerCounterGateTime(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, double gateTime);

    /**
     * @brief Determine the range for a timer/counter channel.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder that the timer/counter channel belongs to (e.g. 'A' for the first slot).
     * @param [in] channelIndex The zero-based index of the timer/counter channel to determine the range for.
     * @param [out] lowerValue The lower range value.
     * @param [out] upperValue The upper range value.
     * @return @ref GHSReturnValue
     *
     * @ingroup TimerCounterChannel
     * @sa @ref GHSSetTimerCounterRange()
     */
    GHSReturnValue GHSGetTimerCounterRange(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, double *lowerValue, double *upperValue);

    /**
     * @brief Set the range for a timer/counter channel.
     *
     * @attention The system needs to be idle before calling this function.\n
     * If the specified timer/counter range is illegal
     * (i.e. upperValue < lowerValue), the timer/counter range values are
     * corrected to the nearest possible values.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder that the timer/counter channel belongs to (e.g. 'A' for the first slot).
     * @param [in] channelIndex The zero-based index of the timer/counter channel to set the range for.
     * @param [in] lowerValue The desired lower range value.
     * @param [in] upperValue The desired upper range value.
     * @return @ref GHSReturnValue
     *
     * @ingroup TimerCounterChannel
     * @sa @ref GHSGetTimerCounterRange()
     */
    GHSReturnValue GHSSetTimerCounterRange(GHSConnectionHandle conHandle, GHSSlotID slotID, int channelIndex, double lowerValue, double upperValue);

    /**
     * @defgroup Sweep Sweep
     * @brief Sweep recording mode related API functions.
     */

    /**
     * @brief Determine the sweep recording mode for a recorder.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder from which to retrieve the sweep recording mode (e.g. 'A' for the first slot).
     * @param [out] sweepRecordingMode The sweep recording mode for the recorder @ref GHSSweepRecordingMode.
     * @return @ref GHSReturnValue
     *
     * @ingroup Sweep
     * @sa @ref GHSSetSweepRecordingMode()
     */
    GHSReturnValue GHSGetSweepRecordingMode(GHSConnectionHandle conHandle, GHSSlotID slotID, GHSSweepRecordingMode *sweepRecordingMode);

    /**
     * @brief Sets the sweep recording mode for a recorder.
     * 
     * @attention The system needs to be idle before calling this function.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder for which to set the sweep recording mode (e.g. 'A' for the first slot).
     * @param [in] sweepRecordingMode The desired sweep recording mode for the recorder @ref GHSSweepRecordingMode.
     * @return @ref GHSReturnValue
     *
     * @ingroup Sweep
     * @sa @ref GHSGetSweepRecordingMode()
     */
    GHSReturnValue GHSSetSweepRecordingMode(GHSConnectionHandle conHandle, GHSSlotID slotID, GHSSweepRecordingMode sweepRecordingMode);

    /**
     * @brief Determine the sweep length in samples for a recorder.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder from which to retrieve the sweep length (e.g. 'A' for the first slot).
     * @param [out] numberOfSamples The sweep length in number of samples.
     * @return @ref GHSReturnValue
     *
     * @ingroup Sweep
     * @sa @ref GHSSetSweepLength()
     */
    GHSReturnValue GHSGetSweepLength(GHSConnectionHandle conHandle, GHSSlotID slotID, int64_t *numberOfSamples);

    /**
     * @brief Sets the sweep length in samples for a recorder.
     *
     * @attention The system needs to be idle before calling this function.
     * 
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder for which to set the sweep length (e.g. 'A' for the first slot).
     * @param [in] numberOfSamples The desired sweep length in number of samples.
     * @return @ref GHSReturnValue
     *
     * @ingroup Sweep
     * @sa @ref GHSGetSweepLength()
     */
    GHSReturnValue GHSSetSweepLength(GHSConnectionHandle conHandle, GHSSlotID slotID, int64_t numberOfSamples);

    /**
     * @brief Determine the number of sweeps for a recorder.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder from which to retrieve the number of sweeps (e.g. 'A' for the first slot).
     * @param [out] numberOfSweeps The number of sweeps for the recorder.
     * @return @ref GHSReturnValue
     *
     * @ingroup Sweep
     * @sa @ref GHSSetNumberOfSweeps()
     */
    GHSReturnValue GHSGetNumberOfSweeps(GHSConnectionHandle conHandle, GHSSlotID slotID, int *numberOfSweeps);

    /**
     * @brief Sets the number of sweeps for a recorder.
     * 
     * @attention The system needs to be idle before calling this function.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder for which to set the number of sweeps (e.g. 'A' for the first slot).
     * @param [in] numberOfSweeps The desired number of sweeps for the recorder.
     * @return @ref GHSReturnValue
     *
     * @ingroup Sweep
     * @sa @ref GHSGetNumberOfSweeps()
     */
    GHSReturnValue GHSSetNumberOfSweeps(GHSConnectionHandle conHandle, GHSSlotID slotID, int numberOfSweeps);

    /**
     * @brief Determine the trigger position percentage for a recorder.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder from which to retrieve the trigger position (e.g. 'A' for the first slot).
     * @param [out] triggerPosition The trigger position in percentage (0% to 100%).
     * @return @ref GHSReturnValue
     *
     * @ingroup Sweep
     * @sa @ref GHSSetTriggerPosition()
     */
    GHSReturnValue GHSGetTriggerPosition(GHSConnectionHandle conHandle, GHSSlotID slotID, double *triggerPosition);

    /**
     * @brief Sets the trigger position percentage for a recorder.
     * 
     * @attention The system needs to be idle before calling this function.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder for which to set the trigger position (e.g. 'A' for the first slot).
     * @param [in] triggerPosition The desired trigger position in percentage (0% to 100%).
     * @return @ref GHSReturnValue
     *
     * @ingroup Sweep
     * @sa @ref GHSGetTriggerPosition()
     */
    GHSReturnValue GHSSetTriggerPosition(GHSConnectionHandle conHandle, GHSSlotID slotID, double triggerPosition);

    /**
     * @brief Determine if sweep stretch is enabled or disabled for a recorder.
     *
     * @deprecated This function is deprecated and is scheduled to be removed from future versions of this API.\n
     * Please use @ref GHSGetSweepTriggerMode() instead.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder from which to retrieve the sweep stretch enabled status (e.g. 'A' for the first slot).
     * @param [out] enabled The sweep stretch enabled status @ref GHSEnableDisable.
     * @return @ref GHSReturnValue
     *
     * @ingroup Sweep
     * @sa @ref GHSSetSweepStretchEnabled()
     */
    GHSReturnValue GHSGetSweepStretchEnabled(GHSConnectionHandle conHandle, GHSSlotID slotID, GHSEnableDisable *enabled);

    /**
     * @brief Enable or disable sweep stretch for a recorder.
     * 
     * @attention The system needs to be idle before calling this function.
     *
     * @deprecated This function is deprecated and is scheduled to be removed from future versions of this API.\n
     * Please use @ref GHSSetSweepTriggerMode() instead.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder for which to enable/disable sweep stretch (e.g. 'A' for the first slot).
     * @param [in] enabled The disired sweep stretch enabled status @ref GHSEnableDisable.
     * @return @ref GHSReturnValue
     *
     * @ingroup Sweep
     * @sa @ref GHSGetSweepStretchEnabled()
     */
    GHSReturnValue GHSSetSweepStretchEnabled(GHSConnectionHandle conHandle, GHSSlotID slotID, GHSEnableDisable enabled);

    /**
     * @brief Retrieve trigger arm enabled status for a recorder.
     *        When enabled, triggers must be armed explicitly before they will be accepted.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [out] enabled Flag to indicate if feature is on.
     * @return @ref GHSReturnValue
     *
     * @ingroup Sweep
     * @sa @ref GHSSetTriggerArmEnabled()
     */
    GHSReturnValue GHSGetTriggerArmEnabled(GHSConnectionHandle conHandle, GHSEnableDisable *enabled);

    /**
     * @brief Enable or disable trigger arm.
     *        When enabled, triggers must be armed explicitly before they will be accepted.
     * 
     * @attention The system needs to be idle before calling this function.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] enabled Flag to indicate if feature is on.
     * @return @ref GHSReturnValue
     *
     * @ingroup Sweep
     * @sa @ref GHSGetTriggerArmEnabled()
     */
    GHSReturnValue GHSSetTriggerArmEnabled(GHSConnectionHandle conHandle, GHSEnableDisable enabled);

    /**
     * @brief Arm the trigger, so that the next trigger will be accepted.
     *        After the next trigger occurred, triggers are disarmed automatically and need to be
     *        armed explicitly again using this function.
     * 
     * @attention The system needs to be acquiring for this function to have any effect.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @return @ref GHSReturnValue
     *
     * @ingroup Sweep
     */
    GHSReturnValue GHSCmdTriggerArm(GHSConnectionHandle conHandle);

    /**
     * @brief Retrieve the current trigger arm state.
     *        This function can be used to synchronize CmdTriggerArm function calls with the user application.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [out] state The current state of trigger arm (e.g. triggers armed or disarmed).
     * @return @ref GHSReturnValue
     *
     * @ingroup Sweep
     * @sa @ref GHSCmdTriggerArm()
     */
    GHSReturnValue GHSGetTriggerArmState(GHSConnectionHandle conHandle, GHSTriggerArmState *state);

    /**
     * @brief Gets the sweep trigger mode for a recorder.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder from which to retrieve the sweep trigger mode (e.g. 'A' for the first slot).
     * @param [out] mode The sweep trigger mode
     * @return @ref GHSReturnValue
     *
     * @ingroup Sweep
     * @sa @ref GHSSetSweepTriggerMode()
     */
    GHSReturnValue GHSGetSweepTriggerMode(GHSConnectionHandle conHandle, GHSSlotID slotID, GHSSweepTriggerMode *mode);

    /**
     * @brief Sets the sweep trigger mode for a recorder.
     *
     * @attention The system needs to be idle before calling this function.
     *            Setting the sweep trigger mode to "stop trigger" disables timeout triggers automatically.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder for which to set the sweep trigger mode for (e.g. 'A' for the first slot).
     * @param [in] mode The sweep trigger mode to be set
     * @return @ref GHSReturnValue
     *
     * @ingroup Sweep
     * @sa @ref GHSGetSweepTriggerMode()
     * @sa @ref GHSGetTimeoutTriggerEnabled()
     */
    GHSReturnValue GHSSetSweepTriggerMode(GHSConnectionHandle conHandle, GHSSlotID slotID, GHSSweepTriggerMode mode);

    /**
     * @brief Retrieve the timeout trigger enabled status.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [out] enabled Flag to indicate if feature is on.
     * @return @ref GHSReturnValue
     *
     * @ingroup Sweep
     * @sa @ref GHSSetTimeoutTriggerEnabled()
     */
    GHSReturnValue GHSGetTimeoutTriggerEnabled(GHSConnectionHandle conHandle, GHSEnableDisable *enabled);

    /**
     * @brief Enables or disables timeout triggers.
     *
     * @attention The system needs to be idle before calling this function.
     *            If the sweep trigger mode is set to "stop trigger" then timeout triggers are automatically disabled.
     *            This function returns @ref GHSReturnValue_Adapted in case enabling timeout triggers is attempted while the current sweep trigger mode is "stop trigger".
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] enabled Flag to indicate if feature is on.
     * @return @ref GHSReturnValue
     *
     * @ingroup Sweep
     * @sa @ref GHSGetTimeoutTriggerEnabled()
     * @sa @ref GHSGetSweepTriggerMode()
     */
    GHSReturnValue GHSSetTimeoutTriggerEnabled(GHSConnectionHandle conHandle, GHSEnableDisable enabled);

    /**
     * @brief Retrieve the timeout trigger time.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] time The timeout trigger time in seconds.
     * @return @ref GHSReturnValue
     *
     * @ingroup Sweep
     * @sa @ref GHSSetTimeoutTriggerTime()
     */
    GHSReturnValue GHSGetTimeoutTriggerTime(GHSConnectionHandle conHandle, double *time);

    /**
     * @brief Sets the timeout trigger time.
     *
     * @attention The system needs to be idle before calling this function.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] time The timeout trigger time in seconds.
     * @return @ref GHSReturnValue
     *
     * @ingroup Sweep
     * @sa @ref GHSGetTimeoutTriggerTime()
     */
    GHSReturnValue GHSSetTimeoutTriggerTime(GHSConnectionHandle conHandle, double time);

    /**
     * @defgroup Continuous Continuous
     * @brief Continuous recording mode related API functions.
     */

    /**
     * @brief Determine the continuous recording mode for a recorder.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder from which to retrieve the continuous recording mode (e.g. 'A' for the first slot).
     * @param [out] continuousMode The continuous recording mode @ref GHSContinuousRecordingMode.
     * @return @ref GHSReturnValue
     *
     * @ingroup Continuous
     * @sa @ref GHSSetContinuousRecordingMode()
     */
    GHSReturnValue GHSGetContinuousRecordingMode(GHSConnectionHandle conHandle, GHSSlotID slotID, GHSContinuousRecordingMode *continuousMode);

    /**
     * @brief Set the continuous recording mode for a recorder.
     * 
     * @attention The system needs to be idle before calling this function.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder for which to set the continuous recording mode (e.g. 'A' for the first slot).
     * @param [in] continuousMode The desired continuous recording mode @ref GHSContinuousRecordingMode.
     * @return @ref GHSReturnValue
     *
     * @ingroup Continuous
     * @sa @ref GHSGetContinuousRecordingMode()
     */
    GHSReturnValue GHSSetContinuousRecordingMode(GHSConnectionHandle conHandle, GHSSlotID slotID, GHSContinuousRecordingMode continuousMode);

    /**
     * @brief Determine the continuous recording time span for a recorder.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder from which to retrieve the continuous time span (e.g. 'A' for the first slot).
     * @param [out] timeSpan The time span in seconds for circular and specified time continuous recording modes @ref GHSContinuousRecordingMode.
     * @return @ref GHSReturnValue
     *
     * @ingroup Continuous
     * @sa @ref GHSSetContinuousTimeSpan()
     */
    GHSReturnValue GHSGetContinuousTimeSpan(GHSConnectionHandle conHandle, GHSSlotID slotID, double *timeSpan);

    /**
     * @brief Set the continuous recording time span for a recorder.
     * 
     * @attention The system needs to be idle before calling this function.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder for which to set the continuous time span (e.g. 'A' for the first slot).
     * @param [in] timeSpan The desired time span in seconds for circular and specified time continuous recording modes @ref GHSContinuousRecordingMode.
     * @return @ref GHSReturnValue
     *
     * @ingroup Continuous
     * @sa @ref GHSGetContinuousTimeSpan()
     */
    GHSReturnValue GHSSetContinuousTimeSpan(GHSConnectionHandle conHandle, GHSSlotID slotID, double timeSpan);

    /**
     * @brief Determine the continuous recording lead out time for a recorder.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder from which to retrieve the continuous recording lead out time (e.g. 'A' for the first slot).
     * @param [out] leadOutTime The continuous recording mode lead out time (post trigger time) in seconds.
     * @return @ref GHSReturnValue
     *
     * @ingroup Continuous
     * @sa @ref GHSSetContinuousLeadOutTime()
     */
    GHSReturnValue GHSGetContinuousLeadOutTime(GHSConnectionHandle conHandle, GHSSlotID slotID, double *leadOutTime);

    /**
     * @brief Sets the continuous recording lead out time for a recorder.
     * 
     * @attention The system needs to be idle before calling this function.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] slotID The slot containing the recorder for which to set the continuous recording lead out time (e.g. 'A' for the first slot).
     * @param [in] leadOutTime The desired continuous recording mode lead out time (post trigger time) in seconds.
     * @return @ref GHSReturnValue
     *
     * @ingroup Continuous
     * @sa @ref GHSGetContinuousLeadOutTime()
     */
    GHSReturnValue GHSSetContinuousLeadOutTime(GHSConnectionHandle conHandle, GHSSlotID slotID,  double leadOutTime);

    /**
     * @defgroup FieldBus FieldBus
     * @brief FieldBus related API functions
     * @warning FieldBus functionality is only provided by tethered mainframes and does not function on integrated mainframes.
     */

    /**
     * @brief Opens a new connection for the field bus data, and waits for client to connect.
     *        Once the client connects, if the acquisition state is active, data is sent to the client. Otherwise nothing is sent.
     *
     * @note If no formulas are present then you get error code back:GHSReturnValue_FieldBusError_NotConfigured
     *       if the field bus is already initiated then an error code is returned:GHSReturnValue_FieldBusError_FieldBusEnabled.
     *       To change the update rate the field bus needs to be stoped and inititate again with a new update rate.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in,out] updateRate (in Hz) The update rate should be a part of the list containing the supported update rates.If the update rate selected by
     *                            the user is not supported, the mainframe will adjust the update rate, based on a minimum distance from the
     *                            supported rates and returns back to the client.
     * @param [out] dataCount The number of published data on the setup.Timestamp is not included in this count.
     * @return @ref GHSReturnValue
     *
     * @ingroup FieldBus
     * @sa @ref GHSReceiveFieldBusData()
     * @sa @ref GHSStopFieldBusDataTransfer()
     */
    GHSReturnValue GHSInitiateFieldBusDataTransfer(GHSConnectionHandle conHandle, int *updateRate, int *dataCount);

    /**
     * @brief Closes the socket dedicated for the field bus data and stops transfering data.
     *        The ring buffer allocated for the transfer data is deleted.
     * 
     * @attention Field bus data transfer must have been initiated before calling this function.
     * See @ref GHSInitiateFieldBusDataTransfer().
     * 
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @return @ref GHSReturnValue
     *
     * @ingroup FieldBus
     * @sa @ref GHSInitiateFieldBusDataTransfer()
     */
    GHSReturnValue GHSStopFieldBusDataTransfer(GHSConnectionHandle conHandle);

    /**
     * @brief Gets the number of data that are configured to be sent through the field bus, excluding the timestamp.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [out] dataCount The number of data that are selected to be published, excluding timestamp.
     * @return @ref GHSReturnValue
     *
     * @ingroup FieldBus
     */
    GHSReturnValue GHSGetFieldBusDataCount(GHSConnectionHandle conHandle, int *dataCount);

    /**
     * @brief Gets the name of specific field bus data value, by giving the index of the data value.
     *        The number of data values can be retrieved by @ref GHSGetFieldBusDataCount().
     *        In case of no formulas (or reserved values acquisition state and latency) published, an error code is returned: GHSReturnValue_FieldBusError_NoFormulasDeployed.
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] dataIndex Zero-based index of the data value.
     * @param [out] dataName Name of the data value.
     * @param [out] dataUnit Unit of the data value.
     * @return @ref GHSReturnValue
     *
     * @ingroup FieldBus
     */
    GHSReturnValue GHSGetFieldBusDataNameAndUnit(GHSConnectionHandle conHandle, int dataIndex, char **dataName, char **dataUnit);

    /**
     * @brief Retrieves the next snapshot in the buffer.
     *       The timestamp is grid aligned - as specified by the 'updateRate' argument of the @ref GHSInitiateFieldBusDataTransfer() method.
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] callType flag to control the receiving of FieldBus data.
     *                         If set to GHS_BlockingCall it blocks while waiting for new FieldBus data.
     *                         If set to GHS_NonBlockingCall then returns last FieldBus data when no new data available.
     *                         GHS_BlockingCall is the preferred value as this results in the lowest CPU load and highest data troughput.
     * @param [out] timestamp timestamp of the FieldBus data relative to start of acquisition @ref GHSGetAcquisitionStartTime().
     * @param [out] values the values published on the FieldBus.
     * @param [out] overrun is one if overrun happened in the buffer, zero if not.
     * @return @ref GHSReturnValue
     *
     * @ingroup FieldBus
     */
    GHSReturnValue GHSReadNextSnapshot(GHSConnectionHandle conHandle, GHSFunctionCallType callType, double *timestamp, float *values, int *overrun);

    /**
     * @brief Used to retrieve the FieldBus data from the new socket connection and write on the buffer. Can be used in a thread,
     *      if so mutual exclusion should be used with GHSStopFieldBusDataTransfer. This function should be used in a loop if continuous
     *      delivery of Async Data is desired.
     * 
     * @attention Field bus data transfer must have been initiated before calling this function.
     * See @ref GHSInitiateFieldBusDataTransfer().
     * 
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     *
     * @ingroup FieldBus
     * @return @ref GHSReturnValue
     */
    GHSReturnValue GHSReceiveFieldBusData(GHSConnectionHandle conHandle);

    /**
     * @brief Request a single snapshot of the field bus data.
     *       In this case the timestamp received is not grid aligned.
     *       If the system is not acquiring the method returns an error code:GHSReturnValue_SystemNotRecording
     *       In case of no formulas you get the error return code back:GHSReturnValue_FieldBusError_NotConfigured
     *       if the field bus is already enabled then this method is not allowed.An error return code is returned:FieldBusError_UnavailableFunctionality
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [out] timestamp of the FieldBus data relative to start of acquisition (not grid aligned) @ref GHSGetAcquisitionStartTime().
     * @param [in,out] dataCount [in] Number of elements allocated in the data parameter by the customer.
     *                           [out] Actuall number of elements(formulas) in the data parameter.
     *                           If number of elements allocated by the customer are less than the actual formulas number, then we return an error code.
     * @param [out] data Array of floats that contains the data
     * @return @ref GHSReturnValue
     *
     * @ingroup FieldBus
     */
    GHSReturnValue GHSRequestFieldBusSnapshot(GHSConnectionHandle conHandle, double *timestamp, int *dataCount, float *data);

    /**
     * @defgroup CANBus CAN fieldbus
     * @brief CAN fieldbus related API functions.
     */

    /**
     * @brief Get the current setup of CAN acquisition control
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [out] enabled Flag to indicate if feature is on
     * @param [out] busID The one-based index that specifies which CAN controller is used
     * @param [out] msgID Message ID of the CAN acquisition control
     * @return @ref GHSReturnValue
     *
     * @ingroup CANBus
     * @sa @ref GHSGetCANAcqControl()
     */
    GHSReturnValue GHSGetCANAcqControl(GHSConnectionHandle conHandle, GHSEnableDisable *enabled, int *busID, int *msgID);

    /**
     * @brief Set up CAN acquisition control
     * 
     * @attention The system needs to be idle before calling this function.
     *
     * @param [in] conHandle The unique connection identifier returned by @ref GHSConnect().
     * @param [in] enabled Flag to indicate if feature is on
     * @param [in] busID The one-based index that specifies which CAN controller to use
     * @param [in] msgID Message ID of the CAN acquisition control
     * @return @ref GHSReturnValue
     *
     * @ingroup CANBus
     * @sa @ref GHSSetCANAcqControl()
     */
    GHSReturnValue GHSSetCANAcqControl(GHSConnectionHandle conHandle, GHSEnableDisable enabled, int busID, int msgID);

#if defined(__cplusplus)
}
#endif

#endif /* GEN_DAQ_API_H */
