/*
* Copyright (C) 2009 HBM Netherlands B.V.
* Schutweg 15a
* 5145 NP Waalwijk
* The Netherlands
* http://www.hbm.com
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
*/

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "connection.h"
#include "socket.h"

#define MAX_CONNECTIONS 30
#define GHS_INVALID_HANDLE 0
#define MAKE_HEADER_VERSION(a,b) (((unsigned char)('G') << 24) | ((unsigned char)('D') << 16) | ((unsigned char)(a) << 8) | (unsigned char)(b))

static const uint32_t api_version_header = MAKE_HEADER_VERSION(0, 1);

typedef struct Connection
{
    socket_t sock;
    GHSConnectionHandle handle;
    uint16_t requestID;
} Connection;


typedef struct PacketHeader
{
    uint32_t length; /* in network order */
    uint32_t packet_type;
} PacketHeader;

static Connection *g_arConnections[MAX_CONNECTIONS] = { NULL };
static uint16_t g_uNextHandle = 1;


static Connection *ConnectionGet(GHSConnectionHandle *handle);
static uint16_t GetNextHandle(void);
static size_t GetConnectionCount(void);

static int ConnectionRead(Connection *pConnection, char *buf, int count);
static int ConnectionWrite(Connection *pConnection, const char *buf, int count);

#ifdef __MINGW32__
WINSOCK_API_LINKAGE int WSAAPI inet_pton(int af, const char *src, void *dst);
#endif

GHSReturnValue ConnectionEstablish(GHSConnectionHandle* handle, const char* ipAddress, unsigned short portNumber)
{
    int err;
    struct sockaddr_in host_addr;
    socket_t sock;
    Connection *pNewConnection = NULL;
    size_t i;

    if (!handle || !ipAddress)
        return GHSReturnValue_NullPtrArgument;

    *handle = GHS_INVALID_HANDLE;

    if (GetConnectionCount() == MAX_CONNECTIONS)
        return GHSReturnValue_ConnectionFailed;

    if (SockInit() < 0)
        return GHSReturnValue_NOK;

    memset(&host_addr, 0, sizeof(host_addr));
    host_addr.sin_family = AF_INET;
    err = inet_pton(AF_INET, ipAddress, &(host_addr.sin_addr));

    if (err == 0) /*1 on succes, 0 on invalid IP string*/
    {
        SockCleanup();
        return GHSReturnValue_InvalidIP;
    }
    else if (err == -1)
    {
        /*This indicates an invalid Address Family is specified, AF_INET is correct*/
        SockCleanup();
        return GHSReturnValue_NOK;
    }

    host_addr.sin_port = htons(portNumber);

    sock = socket(AF_INET, SOCK_STREAM, 0);
    if (sock == INVALID_SOCKET)
    {
        SockCleanup();
        return GHSReturnValue_ConnectionFailed;
    }

    pNewConnection = (Connection *)malloc(sizeof(Connection));
    if (!pNewConnection)
    {
        SockClose(sock);
        SockCleanup();
        return GHSReturnValue_ConnectionFailed;
    }

    if (connect(sock, (struct sockaddr *) &host_addr, sizeof(host_addr)) < 0)
    {
        free(pNewConnection);
        SockClose(sock);
        SockCleanup();
        return GHSReturnValue_ConnectionFailed;
    }

    for (i = 0; i < MAX_CONNECTIONS; i++)
    {
        if (g_arConnections[i] == NULL)
        {
            if (pNewConnection)
            {
                pNewConnection->sock = sock;
                pNewConnection->handle = GetNextHandle();
                pNewConnection->requestID = 0;

                g_arConnections[i] = pNewConnection;
                *handle = pNewConnection->handle;
            }
            break;
        }
    }

    return GHSReturnValue_OK;
}

GHSReturnValue ConnectionSendRequestWaitResponse(GHSConnectionHandle *handle, const char *request, int requestID, JsonRpcNamedParameter **params, size_t *numParams)
{
    int nWriteLen;
    int completeMessageSize;
    GHSReturnValue returnValue;
    PacketHeader header;
    PacketHeader rxHeader;

    char* completeMessage = 0;
    Connection *pConnection = ConnectionGet(handle);

    if (!handle || !request)
    {
        json_free_serialized_string((char *)request);
        return GHSReturnValue_NullPtrArgument;
    }

    if (pConnection == NULL)
    {
        json_free_serialized_string((char *)request);
        return GHSReturnValue_InvalidHandle;
    }

    if (pConnection->sock == INVALID_SOCKET)
    {
        json_free_serialized_string((char *)request);
        return GHSReturnValue_NoConnection;
    }

    if (params)
        *params = 0;
    if (numParams)
        *numParams = 0;

    nWriteLen = (int)(strlen(request) + 1);
    header.packet_type = htonl(api_version_header);
    header.length = htonl((uint32_t)nWriteLen);
    if (ConnectionWrite(pConnection, (const char*)&header, sizeof(header)) != sizeof(header))
        goto error_protocol;

    if (ConnectionWrite(pConnection, request, (int)nWriteLen) != nWriteLen)
        goto error_protocol;

    if (ConnectionRead(pConnection, (char*)&rxHeader, sizeof(rxHeader)) != sizeof(rxHeader))
        goto error_protocol;

    if (ntohl(rxHeader.packet_type) != api_version_header)
        goto error_protocol;

    completeMessageSize = (int)ntohl(rxHeader.length);
    if (completeMessageSize < 1)
        goto error_protocol;

    completeMessage = (char*)malloc(completeMessageSize);
    if (!completeMessage)
        goto error_protocol;

    if (ConnectionRead(pConnection, completeMessage, completeMessageSize) != completeMessageSize)
        goto error_protocol;

    if (completeMessage[completeMessageSize - 1] != '\0')
        goto error_protocol;

    returnValue = JsonRpcParseResponse(completeMessage, requestID, params, numParams);
    goto cleanup;

error_protocol:
    returnValue = GHSReturnValue_NOK;
cleanup:
    if (request)
        json_free_serialized_string((char *)request);
    if (completeMessage)
        free(completeMessage);

    return returnValue;
}

GHSReturnValue ConnectionReadRaw(GHSConnectionHandle *handle, char* raw, int size)
{
    Connection *pConnection = NULL;

    if (!handle || !raw)
        return GHSReturnValue_NullPtrArgument;

    pConnection = ConnectionGet(handle);
    if (!pConnection)
        return GHSReturnValue_InvalidHandle;
    if (pConnection->sock == INVALID_SOCKET)
        return GHSReturnValue_NoConnection;

    if (ConnectionRead(pConnection, raw, size) != size)
        return GHSReturnValue_NOK;

    return GHSReturnValue_OK;
}

GHSReturnValue ConnectionWriteRaw(GHSConnectionHandle *handle, const char* raw, int size)
{
    Connection *pConnection = NULL;

    if (!handle || !raw)
        return GHSReturnValue_NullPtrArgument;

    pConnection = ConnectionGet(handle);
    if (!pConnection)
        return GHSReturnValue_InvalidHandle;
    if (pConnection->sock == INVALID_SOCKET)
        return GHSReturnValue_NoConnection;

    if (ConnectionWrite(pConnection, raw, size) != size)
        return GHSReturnValue_NOK;

    return GHSReturnValue_OK;
}

GHSReturnValue ConnectionClose(GHSConnectionHandle *handle)
{
    size_t i;
    Connection *pConnection = NULL;

    if (!handle)
        return GHSReturnValue_NullPtrArgument;
    if (*handle == GHS_INVALID_HANDLE)
        return GHSReturnValue_InvalidHandle;

    for (i = 0; i < MAX_CONNECTIONS; i++)
    {
        pConnection = g_arConnections[i];
        if ((pConnection != NULL) && (pConnection->handle == *handle))
        {
            g_arConnections[i] = NULL;
            if (pConnection->sock != INVALID_SOCKET)
            {
                shutdown(pConnection->sock, SHUT_WR);
                SockClose(pConnection->sock);
                SockCleanup();
            }
            free(pConnection);
            return GHSReturnValue_OK;
        }
    }

    return GHSReturnValue_InvalidHandle;
}

uint16_t ConnectionNextRequestID(GHSConnectionHandle *handle)
{
    Connection *connection = ConnectionGet(handle);
    if (connection)
    {
        return ++(connection->requestID);
    }
    return 0;
}

Connection *ConnectionGet(GHSConnectionHandle *handle)
{
    size_t i;
    Connection *pConnection;
    if (!handle || handle == GHS_INVALID_HANDLE)
    {
        return NULL;
    }
    for (i = 0; i < MAX_CONNECTIONS; i++)
    {
        pConnection = g_arConnections[i];
        if ((pConnection != NULL) && (pConnection->handle == *handle))
        {
            return pConnection;
        }
    }
    return NULL;
}

uint16_t GetNextHandle(void)
{
    GHSConnectionHandle h = g_uNextHandle;
    do
    {
        g_uNextHandle++;
        if (g_uNextHandle == GHS_INVALID_HANDLE)
        {
            g_uNextHandle++;
        }
        h = g_uNextHandle;
    } while (ConnectionGet(&h));
    return g_uNextHandle;
}

static size_t GetConnectionCount(void)
{
    size_t nCount = 0;
    size_t i;
    for (i = 0; i < MAX_CONNECTIONS; i++)
    {
        if (g_arConnections[i] != NULL)
        {
            nCount++;
        }
    }
    return nCount;
}

/* read a number of bytes, keep reading until all data is available, or until an error occurs */
/* returns: number of bytes read, anything less than count indicates error */
static int ConnectionRead(Connection *pConnection, char *buf, int count)
{
    int bytesRead = 0;
    int totalBytesReceived = 0;
    int remainingBytes = count;
    do
    {
        bytesRead = SockRead(pConnection->sock, buf + totalBytesReceived, remainingBytes);
        remainingBytes -= bytesRead;
        totalBytesReceived += bytesRead;
    } while (bytesRead > 0 && totalBytesReceived < count);
    return totalBytesReceived;
}

/* write  a number of bytes */
/* returns: number of bytes written, anything less than 'count' indicates error */
static int ConnectionWrite(Connection *pConnection, const char *buf, int count)
{
    return SockWrite(pConnection->sock, buf, count);
}

