/*
 * Copyright (C) 2009 HBM Netherlands B.V.
 * Schutweg 15a
 * 5145 NP Waalwijk
 * The Netherlands
 * http://www.hbm.com
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>

#include "GEN_DAQ_API.h"

static void confirm_close(void);
static void terminate(const char *format, ...);

int main(void)
{
    char* ipaddress = "192.168.1.1";
    unsigned short portno = 8006;
    GHSConnectionHandle connection = 0;
    GHSReturnValue returnCode;
    GHSStorageLocation currentLocation = GHSStorageLocation_Reserved;
    GHSStorageLocation desiredLocation = GHSStorageLocation_Local1;
    uint32_t serverAPIVersion;

    printf("Connecting to %s:%u\n", ipaddress, portno);

    /* Connect to the mainframe "localhost" is also possible. */

    returnCode = GHSConnect(ipaddress, portno, &connection, &serverAPIVersion);
    if (returnCode == GHSReturnValue_APIMismatch)
    {
        terminate("Failed: client has API version %u, while mainframe has API version %u.\n",
        GHSGetClientAPIVersion(), serverAPIVersion);
    }
    else if (returnCode != GHSReturnValue_OK)
    {
        terminate("Failed: return code is %d.\n", returnCode);
    }

    printf("Connected\n");

    /*Get storage location*/

    returnCode = GHSGetStorageLocation(connection, &currentLocation);
    if (returnCode != GHSReturnValue_OK)
    {
        terminate("Failed on GHSGetStorageLocation: return code is %d.\n", returnCode);
    }

    printf("\nStorage location %d.\n", currentLocation);

    /*Set storage location*/

    returnCode = GHSSetStorageLocation(connection, desiredLocation);
    if (returnCode != GHSReturnValue_OK)
    {
        terminate("Failed on GHSSetStorageLocation: return code is %d.\n", returnCode);
    }

    printf("\nStorage location %d.\n", desiredLocation);
    
    printf("\nStart preview\n");

    /* Start preview mode (this enables EtherCAT output). */

    returnCode = GHSStartPreview(connection);
    if (returnCode != GHSReturnValue_OK)
    {
        terminate("Failed on GHSStartPreview: return code is %d.\n", returnCode);
    }

    /* Do something while the GHS mainframe is in preview mode (e.g. monitor output data on EtherCAT). */

    printf("\nStop preview\n");

    /* Stop preview mode. */

    returnCode = GHSStopPreview(connection);
    if (returnCode != GHSReturnValue_OK)
    {
        terminate("Failed on GHSStopPreview: return code is %d.\n", returnCode);
    }

    printf("\nStart recording\n");

    /* Start recording.
     * Note that the mainframe should be configured to store recordings on its local disk. */

    returnCode = GHSStartRecording(connection);
    if (returnCode != GHSReturnValue_OK)
    {
        terminate("Failed on GHSStartRecording: return code is %d.\n", returnCode);
    }

    printf("\nPause recording\n");

    /* Do something while the GHS mainframe is recording. */

    /* Pause recording. */

    returnCode = GHSPauseRecording(connection);
    if (returnCode != GHSReturnValue_OK)
    {
        terminate("Failed on GHSPauseRecording: return code is %d.\n", returnCode);
    }

    printf("\nResume recording\n");

    /* Resume recording. */

    returnCode = GHSResumeRecording(connection);
    if (returnCode != GHSReturnValue_OK)
    {
        terminate("Failed on GHSResumeRecording: return code is %d.\n", returnCode);
    }

    printf("\nTrigger\n");

    /* Trigger. */

    returnCode = GHSTrigger(connection);
    if (returnCode != GHSReturnValue_OK)
    {
        terminate("Failed on GHSTrigger: return code is %d.\n", returnCode);
    }

    printf("\nStop recording\n");

    /* Stop recording. */

    returnCode = GHSStopRecording(connection);
    if (returnCode != GHSReturnValue_OK)
    {
        terminate("Failed on GHSStopRecording: return code is %d.\n", returnCode);
    }

    /* Disconnect from the mainframe (this enables Perception or another GHS API client to connect). */

    returnCode = GHSDisconnect(connection);
    if (returnCode != GHSReturnValue_OK)
    {
        terminate("Failed on GHSDisconnect: return code is %d.\n", returnCode);
    }

    printf("\nDisconnected\n");

    confirm_close();
    return EXIT_SUCCESS;
}

void confirm_close(void)
{
    printf("Press ENTER to close this example program.\n");
    (void)getchar();
}

void terminate(const char *format, ...)
{
    va_list args;

    va_start(args, format);
    vprintf(format, args);
    va_end(args);

    confirm_close();

    exit(EXIT_FAILURE);
}
