/*
 * Copyright (C) 2009 HBM Netherlands B.V.
 * Schutweg 15a
 * 5145 NP Waalwijk
 * The Netherlands
 * http://www.hbm.com
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#if defined(__linux__) || defined(__unix__)
#ifndef _POSIX_C_SOURCE
#define _POSIX_C_SOURCE 199309L
#endif
#include <time.h>
#elif defined(_WIN32)
#include <windows.h>
#else
#error Unsupported platform
#endif
#include <string.h>
#include <stdio.h>

#include "fieldbus.h"

/* Advances the FieldBus Ring Buffer writing index */
static void fieldBus_buf_IncrementWritingIndex(FieldBusRingBufHandle rbuf);

/* Advances the FieldBus Ring Buffer reading index */
static void fieldBus_buf_IncrementReadingIndex(FieldBusRingBufHandle rbuf);

/* Checks if the FieldBus Ring Buffer is empty (reading index is the same as writing index) */
static int fieldBus_buf_IsEmpty(FieldBusRingBufHandle handle);

/* Function to initialize some of the FieldBusRingBuf parameters */
static void fieldBus_buf_Reset(FieldBusRingBufHandle handle);

 /* Initialization of the FieldBus Ring Buffer API */
/* returns FieldBusRingBufHandle - handle that points to the FieldBus Ring Buffer */
/* [in] buffer - entries of the FieldBus RingBuffer (type fieldBusData) */
/* [in] size - capacity of the FieldBus RingBuffer, dependent on the FieldBus Update Rate (contains always 100 ms of data history) */
/* [in] dataCount - size of the value array inside each Ring Buffer entry */
FieldBusRingBufHandle fieldBus_buf_Init(size_t size, int dataCount)
{
    unsigned int i = 0;
    FieldBusRingBufHandle rbuf;
    float *vbuf;

    if (size == 0)
    {
        printf("Zero is not an allowed size for the ring buffer \n");
        return NULL;
    }

    /* Allocates memory in the heap for the fieldBusRingBufHandle */
    rbuf = malloc(sizeof(FieldBusRingBuf));
    if (rbuf == NULL)
    {
        printf("Failed allocating memory for the FieldBusRingBuf rbuf \n");
        return NULL;
    }

    rbuf->buffer = malloc(size * sizeof(FieldBusData));
    if (rbuf->buffer == NULL)
    {
        free(rbuf);
        printf("Failed allocating memory for the FieldBusRingBuf rbuf->buffer \n");
        return NULL;
    }

    rbuf->capacity = size;
    rbuf->dataCount = dataCount;

    /* Allocates memory to the fieldBusData value array on each entry of the Ring Buffer */
    vbuf = malloc(size * dataCount * sizeof(float));
    if (vbuf == NULL)
    {
        free(rbuf->buffer);
        free(rbuf);
        printf("Failed allocating memory for the FieldBusRingBuf values \n");
        return NULL;
    }

    for(i = 0; i < size; i++)
    {
       rbuf->buffer[i].fValues = vbuf + (i * dataCount);
    }

    fieldBus_buf_Reset(rbuf);
    return rbuf;

}

/* Function to initialize some of the FieldBusRingBuf parameters */
/* [out] rbuf - handle for the FieldBus Ring Buffer */
void fieldBus_buf_Reset(FieldBusRingBufHandle rbuf)
{
    if (rbuf == NULL)
    {
        printf("Failed resetting FieldBus Handle \n");
        return;
    }

    rbuf->writeIndex = 0;
    rbuf->readIndex = 0;
    rbuf->overrun = 0;
}

/* Destruction of the FieldBus Ring Buffer API (frees the structure memory and its value array) */
/* [out] rbuf - handle for the FieldBus Ring Buffer */
void fieldBus_buf_Free(FieldBusRingBufHandle rbuf)
{
    if (rbuf == NULL)
    {
        printf("Failed freeing FieldBus Ring Buffer \n");
        return;
    }

    free(rbuf->buffer[0].fValues); /* First pointer points to the entire memory block */
    free(rbuf->buffer);
    free(rbuf);
}

/* Checks if the FieldBus Ring Buffer is empty (reading index is the same as writing index) */
/* returns int - 1 if the writing and reading indexes are the same and is not Full, 0 otherwise */
/* [out] rbuf - handle for the FieldBus Ring Buffer */
int fieldBus_buf_IsEmpty(FieldBusRingBufHandle rbuf)
{
    if (rbuf == NULL)
    {
        printf("Failed getting empty status of FieldBus Ring Buffer \n");
        return 0;
    }

    return (!rbuf->overrun && rbuf->writeIndex == rbuf->readIndex);
}

/* Retrieves the FieldBus Ring Buffer size (difference between writing and reading index) */
/* returns size_t - Ring Buffer Current Size */
/* [out] rbuf - handle for the FieldBus Ring Buffer */
size_t fieldBus_buf_GetSize(FieldBusRingBufHandle rbuf)
{
    size_t size;

    if (rbuf == NULL)
    {
        printf("Failed getting size of FieldBus Ring Buffer  \n");
        return 0;
    }

    if (rbuf->writeIndex >= rbuf->readIndex)
    {
        size = (rbuf->writeIndex - rbuf->readIndex);
    }
    else
    {
        size = (rbuf->capacity + rbuf->writeIndex - rbuf->readIndex);
    }

    return size;
}

/* Advances the FieldBus Ring Buffer writing index. This internal function is only used in fieldBus_buf_PutAsyncData */
/* [out] rbuf - handle for the FieldBus Ring Buffer */
static void fieldBus_buf_IncrementWritingIndex(FieldBusRingBufHandle rbuf)
{
    if (rbuf == NULL)
    {
        printf("Failed accessing to the FieldBus Ring Buffer  \n");
        return;
    }

    /* Advances Writing Index */
    rbuf->writeIndex = (rbuf->writeIndex + 1) % rbuf->capacity;

    /* Checks if Writing index is the same as Reading Index after advancing. */
    /* This means the Ring buffer is Full and overrun happened. */
    if(rbuf->writeIndex == rbuf->readIndex)
    {
        rbuf->overrun = 1;
    }
}

/* Advances the FieldBus Ring Buffer reading index. This internal function is only used in fieldBus_buf_GetAsyncData */
/* [out] rbuf - handle for the FieldBus Ring Buffer */
static void fieldBus_buf_IncrementReadingIndex(FieldBusRingBufHandle rbuf)
{
    if (rbuf == NULL)
    {
        printf("Failed accessing to the FieldBus Ring Buffer  \n");
        return;
    }

    rbuf->readIndex = (rbuf->readIndex + 1) % rbuf->capacity;
}
/* Insert FieldBusData in the FieldBus Ring Buffer */
/* [out] rbuf - handle for the FieldBus Ring Buffer */
/* [in] timestamp - timestamp of the values received */
/* [in] latency - latency of the FieldBus */
/* [in] values - FieldBus values */
void fieldBus_buf_PutAsyncData(FieldBusRingBufHandle rbuf, double timestamp, float* values)
{
    if (!(rbuf || rbuf->buffer))
    {
        printf("FieldBus Ring Buffer is null \n");
        return;
    }

    /* Places the fieldBusData on the entry of the Ring Buffer */
    rbuf->buffer[rbuf->writeIndex].dTimestamp = timestamp;

    /* Copies the fieldBusData Array to the Ring Buffer value array entry */
    memcpy(rbuf->buffer[rbuf->writeIndex].fValues, values, rbuf->dataCount * sizeof(float));

    /* Advances the Writing index */
    fieldBus_buf_IncrementWritingIndex(rbuf);
}

/* Read FieldBusData in the FieldBus Ring Buffer */
/* returns Boolean - 1 if Overrun has happened, 0 if not */
/* [out] rbuf - handle for the FieldBus Ring Buffer */
/* [out] timestamp - timestamp of the values received */
/* [out] values - FieldBus values */
/* [in] callType - flag used to wait for new data or to continue execution */
int fieldBus_buf_GetAsyncData(FieldBusRingBufHandle rbuf, double* timestamp, float* values, GHSFunctionCallType callType)
{
    /* Needs to be reviewed */
    if (callType == GHS_BlockingCall)
    {
        while (fieldBus_buf_GetSize(rbuf) == 0)
        {
#if defined(__linux__) || defined(__unix__)
            const struct timespec req = { 0, 100000L };
            nanosleep(&req, NULL);
#elif defined(_WIN32)
            Sleep(1);
#else
#error Unsupported platform
#endif
        }
    }

    if (!(rbuf || rbuf->buffer))
    {
        printf("FieldBus Ring Buffer or the FieldBus data is null\n");
        return 0;
    }

    /* Checks if the Ring Buffer has information available */
    if (!fieldBus_buf_IsEmpty(rbuf))
    {
        *timestamp = rbuf->buffer[rbuf->readIndex].dTimestamp;
        memcpy(values, rbuf->buffer[rbuf->readIndex].fValues, rbuf->dataCount * sizeof(float));
        fieldBus_buf_IncrementReadingIndex(rbuf);
    }

    /* Returns Overrun Flag */
    return rbuf->overrun;
}

/* Read FieldBusData packet from the FieldBus socket connection */
/* returns GHSReturnValue */
/* [in] fieldBusHandle - handle for the FieldBus connection */
/* [out] packet - the read field bus data packet */
/* [in] dataCount - the number of published formulas, including published reserved channels */
GHSReturnValue FieldBusConnectionRead(GHSConnectionHandle *fieldBusHandle, FieldBusData *packet, int dataCount)
{
    GHSReturnValue returnValue = GHSReturnValue_OK;
    int fieldBusPacketSize = sizeof(double) /* timestamp */ + (dataCount * sizeof(float));
    char *fieldBusStream = (char*)malloc(fieldBusPacketSize);

    returnValue = ConnectionReadRaw(fieldBusHandle, fieldBusStream, fieldBusPacketSize);
    if (returnValue != GHSReturnValue_OK)
        goto cleanup;

    memcpy(&packet->dTimestamp, fieldBusStream, sizeof(double)); /* timestamp */
    memcpy(packet->fValues, fieldBusStream + sizeof(double), sizeof(float) * dataCount);

cleanup:
    free(fieldBusStream);
    return returnValue;
}

